/*
cash.h
Copyright (C)2001  Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.
*/

#ifndef _CCASH_H_
#define _CCASH_H_

#include <list>
#include <algorithm>

/**
 * @defgroup commlib Common Libraries
 * @{
 */
 
namespace bhpos{

/**
 * @defgroup bhcash Cash
 * @{
 *
 * \ingroup commlib
 * @{	
 */

/*! \brief A class to handle monetary units. */
class CMoney{
public:
	int	 unit;      //! with reference to 1
	int 	 amount;    //! number of integer units of that amount
	double value;     //! the value of the unit with reference to 1.0
	
public:
	CMoney();
	CMoney(const CMoney&);
	CMoney(int u, double v, double a);
	CMoney(int u, double v, int a);
	virtual ~CMoney();
	CMoney &operator=(const CMoney&);
	bool operator==(const CMoney&);
	bool operator!=(const CMoney&);
	bool operator==(const double&);
	bool operator!=(const double&);
	bool operator<(const CMoney&);
	bool operator<(const double&);
	bool operator>(const CMoney&);
	bool operator>(const double&);
	
	void	 Set(int u, double v, double a);
	void	 Set(int u, double v, int a);
	int	 SetAmount(double);
	int 	 SetAmount(int);
	int	 SetValue(double);
	double GetAmount();
	double GetValue();
	int 	 GetNumber();
	int 	 GetUnit();
	int 	 GetQuantity(double);
	int    Add(double a);
	int    Add(int a);
	int    Subtract(double a);
	int    Subtract(int a);
};

/* @}
 * \ingroup bhcash
 * @{
 */
/*! \brief A class to handle a collection of monetary units. */
class CCash
{
private:
	std::list<CMoney> current;
	int               total_units;
	double				smallest_unit;

protected:
	void   Total();
   
public:
	CCash();
	CCash(const CCash&);
	virtual ~CCash();
	CCash &operator=(const CCash&);
	bool  operator==(CCash&);
	bool  operator!=(CCash&);
	bool  operator<(CCash&);
   void operator+=(CCash&);
   void operator-=(CCash&);
	
	int Clear();
	int Add(CMoney&);
	int Add(int u, double v, int a);
	int Update(CMoney&);
	int AdjustAmount(int u, double v, int a);
	int NumMoneyUnits();
	int GetMoneyUnit(int i, CMoney&);
	double GetAmount(int u, double v);
	double GetTotal();
	float  Remove(float amount);
	float  AddTo(float amount);
	int    GetPhyscialNumber(int u, double v);
};

/* @} 
	@} */

}

/* @} */

#endif // _CCASH_H_
