/*
cfg.cpp - a config file reader.
Copyright (C) 2001  Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 
	The author takes no responsibility for anything at all, even pikachu.
	
*/
#include <cstring>
#include <cstdio>
#include <cstring>
#include <cerrno>
#include <cstdlib>
#include <math.h>
#include <sys/types.h>
#include <ctype.h>
#include <sys/utsname.h>
#include "cfg.h"

namespace BhPos{

CConfig::CConfig()
{
	bInDefault=true;
	bInNode=false;
	hFile=NULL;
}

CConfig::~CConfig()
{
	fclose(hFile);
}

bool CConfig::Load(char *fname)
{
	sDefaultValue[0]  = '\0';
   sSpecificValue[0] = '\0';

   if((hFile=fopen(fname,"r"))==NULL){
      char  buff[64];
      sprintf(buff,"Error opening config file [%s]", fname);
      perror(buff);
      return true;
   }

//    uname(&unamebuf);
//    strcpy(sNodeName,unamebuf.nodename);

   return false;
}

bool CConfig::Open(char *fname)
{
	sDefaultValue[0]  = '\0';
   sSpecificValue[0] = '\0';

   if((hFile=fopen(fname,"wr"))==NULL){
      char  buff[64];
      sprintf(buff,"Error opening config file [%s]", fname);
      perror(buff);
      return true;
   }
   return false;
}

bool CConfig::ParseRecord(char *rec, char *name, char *value)
{
	char *pBuffer;
	
	if(!hFile) return true;
	
	fseek(hFile, 0, SEEK_SET);
	strcpy(sRecName, rec);
	strcpy(sKeyName, name);
	
	while((pBuffer=GetNextRec())!=NULL){
		ParseLine(pBuffer, value);
	}
	/*
	if(pBuffer==NULL) return true;
	strcpy(sRecName, rec);
	strcpy(sKeyName, name);
	if(ParseLine(pBuffer, value)) return true;*/
	return false;
}

bool CConfig::WriteRecord(char *rec, char *name, char *value)
{
   fpos_t  pos;
	
	if(!hFile) return true;
	
	fseek(hFile, 0, SEEK_SET);
	strcpy(sRecName, rec);
	strcpy(sKeyName, name);
	
	while(GetNextToRec(&pos)!=NULL){	   
		ParseToLine(value, pos);
	}
}

bool CConfig::ParseString(char *rec, char *name, char *value)
{
	char pvalue[1024];
	int  y;
	
	memset(pvalue, 0, sizeof(pvalue));
	if(!ParseRecord(rec, name, pvalue)){
		strcpy(value, pvalue);
	}
	else{
		return true;
	}
	return false;
}

bool CConfig::ParseString(char *rec, char *name, char *value, char *defval)
{
	char pvalue[1024];
	int  y;
	
	memset(pvalue, 0, sizeof(pvalue));
	if(!ParseRecord(rec, name, pvalue)){
		strcpy(value, pvalue);
	}
	else{
		strcpy(value, defval);
	}
	return false;
}

bool CConfig::ParseInt(char *rec, char *name, int *value)
{
	char pvalue[32];
	int  y;
	
	memset(pvalue, 0, sizeof(pvalue));
	if(!ParseRecord(rec, name, pvalue)){
		*value=atoi(pvalue);
	}
	else{
		return true;
	}
	return false;
}

bool CConfig::ParseDouble(char *rec, char *name, double *value)
{
	char pvalue[32];
	int  y;
	
	memset(pvalue, 0, sizeof(pvalue));
	if(!ParseRecord(rec, name, pvalue)){
		*value=atof(pvalue);
	}
	else{
		return true;
	}
	return false;
}

bool CConfig::ParseBool(char *rec, char *name, bool *value)
{
	char pvalue[32];
	bool b=false;
	
	memset(pvalue, 0, sizeof(pvalue));
	if(!ParseRecord(rec, name, pvalue)){
		if(*pvalue=='Y' || *pvalue=='y')
			b=true;
		else
			b=false;
	}
	else{
		return true;
	}
	return false;
}

bool CConfig::ParseLine(char *pBuffer, char *value)
{
    char buf[RECSIZE];

    if(pBuffer[0] == '['){
        char *p1 = pBuffer + 1;
        char *p2 = buf;

        // copy the record to a buffer
        while(*p1 && *p1 != ']')
            *p2++ = *p1++;
        *p2 = '\0';

        if(strcasecmp(buf,"default") == 0){
            bInDefault=true;
            bInNode=false;
        }
        else{
            bInDefault=false;
            if(strcasecmp(buf,sRecName) == 0)
                bInNode=true;
            else
                bInNode=false;
        }
    }
    else{
        if(bInNode || bInDefault){
            char *eqp;    // pointer to the '=' within a string
            if(eqp=index(pBuffer,'=')){
                char sKeyword[RECSIZE];
                char sValue[RECSIZE];
                char *p1 = pBuffer;
                char *p2;
                p2 = sKeyword;

                // Isolate the keyword from the value
                while(p1 < eqp){
                    *p2++ = *p1++;
                }
                *p2 = '\0';                
                Strim(sKeyword);
                if(strcasecmp(sKeyword, sKeyName) == 0){
                    eqp++;   // Skip over the equal sign
                    strcpy(sValue,eqp);
                    Strim(sValue);
                    if(bInNode)
                        strcpy(sSpecificValue, sValue);                        
                    else
                        strcpy(sDefaultValue, sValue);
                     
					strcpy(value, sValue);
                }
            }
        }
    }
    return false;
}

bool CConfig::ParseToLine(char *value, fpos_t pos)
{
   char buf[RECSIZE];
   char temp;
   
   temp=fgetc(hFile);
   
   if(temp=='['){
      char p1 = fgetc(hFile);
      char *p2 = buf;

      // parse over record
      while(p1 != ']')
         *p2++=p1=fgetc(hFile);

      *p2 = '\0';

      if(strcasecmp(buf,"default") == 0){
         bInDefault=true;
         bInNode=false;
      }
      else{
         bInDefault=false;
         if(strcasecmp(buf,sRecName) == 0)
           bInNode=true;
         else
           bInNode=false;
      }
   }
   else{
      if(bInNode || bInDefault){
         char sKeyword[RECSIZE];
         char sValue[RECSIZE];
            
         char p1 = fgetc(hFile);
         int  x=0;

         // Isolate the keyword from the value
         while(p1!='='){
            sKeyword[x]=p1;            
            p1=fgetc(hFile);
            x++;    
         }
         sKeyword[x]=0;
         Strim(sKeyword);
         
         if(strcasecmp(sKeyword, sKeyName)==0){
            p1=fgetc(hFile); // Skip over the equal sign
            x=strlen(value);
            if(x){
               fputs(value, hFile);
               // see if the old record was longer
               p1=fgetc(hFile);
               while(p1!=0 || p1!=' ' || p1!='\n' || p1!='#'){
                  fputc(' ', hFile);
               }
            }
         }
      }
   }
   return false;
}

char *CConfig::GetNextRec()
{
   static char   sInBuf[RECSIZE];
   bool	bEof     = false;
   bool	bHaveRec = false;

   while(!bEof && !bHaveRec){
      if(fgets(sInBuf, RECSIZE-1, hFile) == NULL){
         bEof = true;
      }
      else{
         Strim(sInBuf);
         if(*sInBuf)	// If the record has a length of zero, skip it
            bHaveRec = true;
      }
   }

   if(bHaveRec)
       return(sInBuf);
   else
       return(NULL);
}

char *CConfig::GetNextToRec(fpos_t *pos)
{
   static char   sInBuf[RECSIZE];
   bool	bEof     = false;
   bool	bHaveRec = false;

   while(!bEof && !bHaveRec){
      fgetpos(hFile, pos);
      if(fgets(sInBuf, RECSIZE-1, hFile)==NULL){
         bEof=true;
      }
      else{
         Strim(sInBuf);
         if(*sInBuf)	// If the record has a length of zero, skip it
            bHaveRec=true;
      }
   }

   if(bHaveRec)
       return(sInBuf);
   else
       return(NULL);
}

void CConfig::Strim(char *str)
{
   char *p;
   int  nQuoteCount=0;

   // Trim leading spaces

   while(isspace(*str)){
      p = str;
      while(*p){
         *p = *(p+1);
         p++;
      }
   }


   p = str;
   while(*p){
      if(*p == '\"')
         nQuoteCount++;
      if(*p == '#' && (nQuoteCount % 2)==0)
         *p = '\0';      // replace '#' with NULL
      else
         p++;
   }
	p--;
   while(p >= str && isspace(*p)){
      *p = '\0';          // Replace the space with a NULL
      p--;
	}
}


}
