/*
bhdev.cpp
Copyright (C)2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.
*/

//#include "pos.h"
#include <glibmm/iochannel.h>
#include "bhdev.h"

namespace bhpos{

CBhDevice::CBhDevice(BhDevType t)
{
	devname="";
	type=t;
	
	bRetry=true;
	retries=0;
	max_retries=10;
	bBackoff=true;
	backoff_value=0.5;
	
	fd=-1;
	bEnabled=false;
	bTimeout=false;
	timeoutval=10000;
	
	bherror=BHDEV_ERR_NONE;
	error=0;
}

void CBhDevice::operator=(CBhDevice const& dev)
{
   devname=dev.devname;
	type=dev.type;
	
	bRetry=dev.bRetry;
	retries=dev.retries;
	max_retries=dev.max_retries;
	bBackoff=dev.bBackoff;
	backoff_value=dev.backoff_value;
	
	fd=dev.fd;
	bEnabled=dev.bEnabled;
	bTimeout=dev.bTimeout;
	timeoutval=dev.timeoutval;
	
	bherror=dev.bherror;
	error=dev.error;
}

bool CBhDevice::operator==(CBhDevice const& dev)
{
   if(fd==dev.fd) return true;
   return false;
}

bool CBhDevice::operator!=(CBhDevice const& dev)
{
   if(fd=dev.fd) return false;
   return true;
}

CBhDevice::~CBhDevice()
{
	Close();
}

int CBhDevice::Open(Glib::ustring dev, BhDevNotifyType nt=BHDEV_NOTIFY_READ)
{
	int  ret;
	struct stat buf;
   
   notifytypes=nt;

   if(type==BHDEV_TYPE_USB){
      return usbFind(atoi(dev.data()));
   }
   
	if(dev==""){
		bherror=BHDEV_ERR_DEVNULLPARAM;
		return 1;
	}
	
	if(fd!=-1){
		Close();
	}
	
   devname=dev;
	
	return privopen();
}

int CBhDevice::privopen()
{
	if(type==BHDEV_TYPE_SERIAL){
	   struct termios   tio;
	     
	   // standard settings
	   fd=open(devname.data(), O_RDWR | O_NOCTTY | O_EXCL | O_NONBLOCK);
   	if(fd==-1){
		   error=errno;
   		return 3;
	   }
	   tcgetattr(fd, &old_tio);
	   tcgetattr(fd, &tio);
	   
	   tio.c_cflag = CS8 | CLOCAL | CREAD;
	   tio.c_iflag = IGNPAR;
	   tio.c_oflag = 0;
	   tio.c_lflag = 0;
	   tio.c_cc[VMIN] = 1;
	   tio.c_cc[VTIME] = 0;

   	cfsetispeed(&tio, B9600);
	   cfsetospeed(&tio, B9600);

   	tcflush(fd, TCIFLUSH);
	   tcsetattr(fd, TCSANOW, &tio);
	}
	else if(type==BHDEV_TYPE_PARALLEL || type==BHDEV_TYPE_DESCRIPTOR){
	   fd=open(devname.data(), O_RDWR | O_NONBLOCK);
	   if(fd==-1){
   		error=errno;
   		return 4;
	   }
	}
   else if(type==BHDEV_TYPE_NETWORK){
      return 5;
   }
   else{
      return 5;
   }

	return 0;
}


void CBhDevice::Close()
{
	devname="";
	
   /*
	if(timer_tag) gtk_timeout_remove(timer_tag);  
	timer_tag=0;
	
	if(fd_tag) gdk_input_remove(fd_tag);
	fd_tag=0;
   */
   if(type==BHDEV_TYPE_SERIAL){
      tcflush(fd, TCIFLUSH);
	   tcsetattr(fd, TCSANOW, &old_tio);
   }
   else if(type==BHDEV_TYPE_USB){
      usbClose();
      return;
   }

	if(fd!=-1) close(fd);
	fd=-1;
}

bool wobbler(Glib::IOCondition ioc)
{
   return false;
}

void CBhDevice::SetSignals()
{
   Glib::IOCondition ioc;

   if(notifytypes & BHDEV_NOTIFY_TIMEOUT){
   }
   if(notifytypes & BHDEV_NOTIFY_READ){
      ioc&=Glib::IO_IN;
   }
	if(notifytypes & BHDEV_NOTIFY_WRITE){
      ioc&=Glib::IO_OUT;
   }
   if(notifytypes & BHDEV_NOTIFY_PRIORITY){
      ioc&=Glib::IO_PRI;
   }
	if(notifytypes & BHDEV_NOTIFY_HUP){
      ioc&=Glib::IO_HUP;
   }
	if(notifytypes & BHDEV_NOTIFY_ERROR){
      ioc&=Glib::IO_ERR;
      ioc&=Glib::IO_NVAL;
   }
   if(type==BHDEV_TYPE_DESCRIPTOR || type==BHDEV_TYPE_SERIAL || type==BHDEV_TYPE_PARALLEL || type==BHDEV_TYPE_NETWORK){
      if(fd!=-1){
         Glib::RefPtr<Glib::IOChannel> dchan=Glib::IOChannel::create_from_fd(fd);
         connection=Glib::signal_io().connect(sigc::mem_fun(*this, &CBhDevice::Active), dchan, ioc);
      }
   }
}

void CBhDevice::SetParameter(BhDevParam p, void *d)
{
	int  n;
	char c;
	bool b;
	
	switch(p){
	case BHDEV_PARAM_TYPE:
		type=*((BhDevType *) d);
		break;
		
	case BHDEV_PARAM_DEVICE:
      /*
		if(d==NULL){
			bherror=BHDEV_ERR_DEVNULLPARAM;
			return;
		}
	
		if(fd!=-1){
			Close();
		}	
		if(devname) delete devname;
   	devname = new char [strlen((char *) d)+1];
   	if(!devname){
   		bherror=BHDEV_ERR_DEVNAMEALLOC;
   		return;
   	}
   	strcpy(devname, (char *) d);
		popen();*/
	   break;
	   
	case BHDEV_PARAM_ENABLE:
      /*
		bEnabled=*((bool *) d);
		if(bEnabled){
			fd_tag=gdk_input_add(fd, GDK_INPUT_READ, fdActive, this);
		}
		else{
			if(fd_tag) gdk_input_remove(fd_tag);
			fd_tag=0;
		}
		if(bTimeout){
			timer_tag=gtk_timeout_add(timeout, &toActive, this);
		}
		if(type==BHDEV_TYPE_SERIAL){
         int set;

      	ioctl(fd, TIOCMGET, &set);
      	if(bEnabled)
            set|=TIOCM_DTR;
      	else 
      		set&=~TIOCM_DTR;
         ioctl(fd, TIOCMSET, &set);
		}*/
		break;
	case BHDEV_PARAM_TOENABLE:
		bTimeout=*((bool *) d);
		break;
	case BHDEV_PARAM_TOVALUE:
		timeoutval=*((int *) d);
		break;
	case BHDEV_PARAM_TORETRIES:
		max_retries=*((int *) d);
		break;
	case BHDEV_PARAM_TOBACKOFF:
		bBackoff=*((bool *) d);
		break;
	case BHDEV_PARAM_TOBOVALUE:
		backoff_value=*((double *) d);
		break;
	case BHDEV_PARAM_SERIALSOFTFLOW:
	   if(type!=BHDEV_TYPE_SERIAL) return;
		break;
		
	case BHDEV_PARAM_SERIALSPEED:
	{
	   if(type!=BHDEV_TYPE_SERIAL) return;
	   if(fd==-1) return;
	   
	   struct termios   tio;
	   speed_t speed=(*(speed_t *) d);
	   
	   if(tcgetattr(fd, &tio)==-1) return;

   	cfsetispeed(&tio, speed);
	   cfsetospeed(&tio, speed);

   	tcflush(fd, TCIFLUSH);
	   tcsetattr(fd, TCSANOW, &tio);
	}
		break;
		
	case BHDEV_PARAM_SERIALBITS:
   {
	   if(type!=BHDEV_TYPE_SERIAL) return;
	   if(fd==-1) return;
	   
	   tcflush(fd, TCIOFLUSH);
	
      struct termios tio;
      if(tcgetattr(fd, &tio)==-1) return;
      
      int bits=(*(int *) d);

      if(bits==8){
         tio.c_cflag&=~CSIZE;
         tio.c_cflag|=CS8;
         tio.c_iflag&=~ISTRIP;
      } 
   	else if(bits==7){
   		tio.c_cflag&=~CSIZE;
   		tio.c_cflag|=CS7;
   		tio.c_iflag|=ISTRIP;
   	} 
   	else if(bits==6){
   		tio.c_cflag&=~CSIZE;
   		tio.c_cflag|=CS6;
   		tio.c_iflag|=ISTRIP;
   	} 
   	else if(bits==5){
   		tio.c_cflag&=~CSIZE;
   		tio.c_cflag|=CS5;
   		tio.c_iflag|=ISTRIP;
   	} 
	   else{
   		return;
   	}
   	tcsetattr(fd, TCSANOW, &tio);
   }
		break;
		
	case BHDEV_PARAM_SERIALSTOPBITS:
	{
	   if(type!=BHDEV_TYPE_SERIAL) return;
	   if(fd==-1) return;
	   
	   tcflush(fd, TCIOFLUSH);
	
      struct termios tio;
      if(tcgetattr(fd, &tio)==-1) return;
      
      int stopbits=(*(int *) d);
      
	   if(stopbits==2){
   		tio.c_cflag|=CSTOPB;
	   } 
   	else if(stopbits==1){
		   tio.c_cflag&=~CSTOPB;
   	} 
	   else{
		   return;
		}
		tcsetattr(fd, TCSANOW, &tio);
	}
	   
		break;
		
	case BHDEV_PARAM_SERIALPARITY:
	{
	   if(type!=BHDEV_TYPE_SERIAL) return;
	   if(fd==-1) return;
	   
	   tcflush(fd, TCIOFLUSH);
	
      struct termios tio;
      if(tcgetattr(fd, &tio)==-1) return;
      
      BhDevSerialParity parity=(*(BhDevSerialParity *) d);
 
      if(parity==BHDEV_SERIAL_PARITY_NONE){
		   tio.c_cflag&=~PARENB;
   		tio.c_iflag&=~INPCK;
   	} 
   	else if(parity==BHDEV_SERIAL_PARITY_EVEN){
   		tio.c_cflag|=PARENB;
   		tio.c_cflag&=~PARODD;
   		tio.c_iflag|=INPCK;
   	}
   	else if(parity==BHDEV_SERIAL_PARITY_ODD){
		   tio.c_cflag|=PARENB;
   		tio.c_cflag|=PARODD;
		   tio.c_iflag|=INPCK;
   	} 
   	else{
		   return;
   	}
   	tcsetattr(fd, TCSANOW, &tio);
   }
		break;
	
	case BHDEV_PARAM_SERIALBITSET:
	{
	   if(type!=BHDEV_TYPE_SERIAL) return;
	   if(fd==-1) return;
	   
	   BhDevSerialControl bit= (*(BhDevSerialControl *) d);
	   
	   int setbit=0;
	   
	   switch(bit){
      case BHDEV_SERIAL_CTRL_DTR: setbit=TIOCM_DTR; break;
      case BHDEV_SERIAL_CTRL_RTS: setbit=TIOCM_RTS; break;
      case BHDEV_SERIAL_CTRL_CTS: setbit=TIOCM_CTS; break;
      case BHDEV_SERIAL_CTRL_DSR: setbit=TIOCM_DSR; break;
      case BHDEV_SERIAL_CTRL_CD:  setbit=TIOCM_CD; break;
      case BHDEV_SERIAL_CTRL_RI:  setbit=TIOCM_RI; break;
      case BHDEV_SERIAL_CTRL_RX:  setbit=TIOCM_SR; break;
      case BHDEV_SERIAL_CTRL_TX:  setbit=TIOCM_ST; break;
      default: return;
      }
      ioctl(fd, TIOCMBIS, &setbit);      
   }
      break;
      
   case BHDEV_PARAM_SERIALBITCLR:
	{
	   if(type!=BHDEV_TYPE_SERIAL) return;
	   if(fd==-1) return;
	   
	   BhDevSerialControl bit= (*(BhDevSerialControl *) d);
	   
	   int setbit=0;
	   
	   switch(bit){
      case BHDEV_SERIAL_CTRL_DTR: setbit=TIOCM_DTR; break;
      case BHDEV_SERIAL_CTRL_RTS: setbit=TIOCM_RTS; break;
      case BHDEV_SERIAL_CTRL_CTS: setbit=TIOCM_CTS; break;
      case BHDEV_SERIAL_CTRL_DSR: setbit=TIOCM_DSR; break;
      case BHDEV_SERIAL_CTRL_CD:  setbit=TIOCM_CD; break;
      case BHDEV_SERIAL_CTRL_RI:  setbit=TIOCM_RI; break;
      case BHDEV_SERIAL_CTRL_RX:  setbit=TIOCM_SR; break;
      case BHDEV_SERIAL_CTRL_TX:  setbit=TIOCM_ST; break;
      default: return;
      }
      ioctl(fd, TIOCMBIC, &setbit);
      
   }
	   break;
	}
}
void CBhDevice::GetParameter(BhDevParam p, void *d)
{
	switch(p){
	case BHDEV_PARAM_ENABLE:
		*((bool *) d)=bEnabled;
		break;
	case BHDEV_PARAM_TOENABLE:
		*((bool *) d)=bEnabled;
		break;
	case BHDEV_PARAM_TOVALUE:
		*((int *) d)=timeoutval;
		break;
	case BHDEV_PARAM_TORETRIES:
		*((int *) d)=max_retries;
		break;
	case BHDEV_PARAM_TOBACKOFF:
		*((bool *) d)=bBackoff;
		break;
	case BHDEV_PARAM_TOBOVALUE:
		*((double *) d)=backoff_value;
		break;
	
	case BHDEV_PARAM_SERIALBITGET:
	{
	   if(type!=BHDEV_TYPE_SERIAL) return;
	   if(fd==-1) return;
	   
	   int setbit=0;
	   
	   ioctl(fd, TIOCMGET, &setbit);
	   *((int *) d)=setbit;
      
   }
	   break;
	}
}

bool CBhDevice::Timeout()
{
	timeout.disconnect();
	
	if(bRetry){
		retries++;
		if(retries==max_retries){
			// failure...
			timeout_signal.emit();
			return true;
		}
		if(bBackoff)			
			timeoutval+=(int)((float) timeoutval * backoff_value);
	      Glib::signal_timeout().connect(mem_fun(*this, &CBhDevice::Timeout), timeoutval);
		return true;
	}
	return false;
}

bool CBhDevice::Active(Glib::IOCondition ios)
{
	if((ios & Glib::IO_IN)!=0){
      if(notifytypes & BHDEV_NOTIFY_READ){
         read_signal.emit(READ_NORMAL);
      }
   }
      
   if((ios & Glib::IO_OUT)!=0){
      if(notifytypes & BHDEV_NOTIFY_WRITE){
         write_signal.emit();
      }
   }
      
   if((ios & Glib::IO_PRI)!=0){
      if(notifytypes & BHDEV_NOTIFY_PRIORITY){
         read_signal.emit(READ_PRIORITY);
      }
   }
      
   if((ios & Glib::IO_ERR)!=0){
      if(notifytypes & BHDEV_NOTIFY_ERROR){
         error_signal.emit(ERROR_ERROR);
      }
   }
      
   if((ios & Glib::IO_HUP)!=0){
      if(notifytypes & BHDEV_NOTIFY_HUP){
         error_signal.emit(ERROR_HUP);
      }
   }
      
   if((ios & Glib::IO_NVAL)!=0){
      if(notifytypes & BHDEV_NOTIFY_ERROR){
         error_signal.emit(ERROR_NVAL);
      }
	}
	return true;
}

int CBhDevice::Read(int flags, int len, void *data)
{
   switch(type){
   case BHDEV_TYPE_NONE:
      return -1;
   
	case BHDEV_TYPE_DESCRIPTOR:
	case BHDEV_TYPE_SERIAL:
	case BHDEV_TYPE_PARALLEL:
      if(fd==-1) return -1;
	   return read(fd, data, len);
      
	case BHDEV_TYPE_USB:
	case BHDEV_TYPE_NETWORK:
	case BHDEV_TYPE_FIREWIRE:
      break;
   }
	return 0;
}

int CBhDevice::Write(int flags, int len, void *data)
{
   switch(type){
   case BHDEV_TYPE_NONE:
      return -1;
   
	case BHDEV_TYPE_DESCRIPTOR:
	case BHDEV_TYPE_SERIAL:
	case BHDEV_TYPE_PARALLEL:
      if(fd==-1) return -1;
	   return write(fd, data, len);
 
	case BHDEV_TYPE_USB:
	case BHDEV_TYPE_NETWORK:
	case BHDEV_TYPE_FIREWIRE:
      break;
   }
   return 0;
}

int CBhDevice::Control(int flags)
{
   int setbit=0;
   
   if(type!=BHDEV_TYPE_SERIAL) return -1;
   
   setbit=TIOCM_CTS; 
   ioctl(fd, TIOCMBIS, &setbit);
}

int CBhDevice::Status(int flags)
{
   int readbit=0;
   
   if(type!=BHDEV_TYPE_SERIAL) return -1;
      
   if(ioctl(fd, TIOCMGET, &readbit) == 0){
      readbit&=TIOCM_RNG;   // RI
      return readbit;
   }
   else{
      return -1;
   }
}

int CBhDevice::Monitor(int, int, void *)
{
}

int CBhDevice::GetError()
{
	if(error!=0) return error; 
	if(bherror!=0) return bherror;
	return 0;
}

char *CBhDevice::GetErrorString(int err)
{
	switch(err){
	case ENOENT: return "File doesn't exist.";
	}
	return NULL;
}

int CBhDevice::usbFind(int type)
{
   struct usb_bus *busses;
   struct usb_bus *bus;
   int    c, i, a, err;
   
   usb_init();
   usb_find_busses();
   usb_find_devices();
    
   busses = usb_get_busses();
   for(bus = busses; bus; bus = bus->next){
      struct usb_device *dev;
    	for(dev=bus->devices; dev; dev=dev->next){
    		// Check if this device is a printer 
    		if (dev->descriptor.bDeviceClass == type){
    			// Open the device, claim the interface and do your processing 
    			if(usb_dev=usb_open(dev)) return 1;
            
            if((err = usb_claim_interface(usb_dev, 0))) {
	            printf("usb_claim_interface returned an error (%d)\n", err);
            }

            if((err = usb_set_configuration(usb_dev, 1))) {
            	printf("usb_set_configuration returned an error (%d)\n", err);
            }

            if((err = usb_set_altinterface(usb_dev, 0))) {
	            printf("usb_set_altinterface returned an error (%d)\n", err);
            }
    		}
    	}
   }
}

void CBhDevice::usbClose()
{
   if(usb_release_interface(usb_dev, 0)) {
	   printf("usb_release_interface returned an error!\n");
   }
   usb_close(usb_dev);
}

int CBhDevice::usbWrite(int flags, int len, void *data)
{
   return usb_bulk_write(usb_dev, BULK_WRITE_ENDPOINT, (char *) data, len, timeoutval);
}

int CBhDevice::usbRead(int flags, int len, void *data)
{
   return usb_bulk_read(usb_dev, BULK_READ_ENDPOINT, (char *) data, len, timeoutval);
}

}
