/*
bhdevparser.cpp
Copyright (C)2002  Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.

*/
#include <cstring>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>

#include "bhdev.h"

namespace bhpos{

namespace parser{

CDeviceParser::CDeviceParser()
{
	name=NULL;
	manufacturer=NULL;
	model=NULL;
	filename=NULL;
	protocol=NULL;
	ns=NULL;
	
	error=0;
}

CDeviceParser::~CDeviceParser()
{
	if(name) delete name;
	if(manufacturer) delete manufacturer;
	if(model) delete model;
	if(filename) delete filename;
	if(protocol) delete protocol;
}

int CDeviceParser::ParseDevice(xmlDocPtr doc, xmlNsPtr nsp, xmlNodePtr node, CBhDevice& dev)
{
	xmlChar 	*buff;
	int			temp;
	BhDevType	type;
	
	ns=nsp;
	if(ns==NULL){
		error=1;
		return 1;
   }
   
   if(node==NULL){
   	error=2;
		return 2;
   }

	node=node->xmlChildrenNode;
   while(node!=NULL){
      if((!xmlStrcmp(node->name, (const xmlChar *) "device")) && (node->ns == ns)){
	      buff=xmlGetProp(node, (const xmlChar *) "name");
			name=strdup((char *) buff);
		
			buff=xmlGetProp(node, (const xmlChar *) "dev");
			dev.SetParameter(BHDEV_PARAM_DEVICE, (char *) buff);
			
			buff=xmlGetProp(node, (const xmlChar *) "type");
			if(!xmlStrcmp(buff, (const xmlChar *) "descriptor")){
				type=BHDEV_TYPE_DESCRIPTOR;
				dev.SetParameter(BHDEV_PARAM_TYPE, &type);
			}
			else if(!xmlStrcmp(buff, (const xmlChar *) "serial")){
				type=BHDEV_TYPE_SERIAL;
				dev.SetParameter(BHDEV_PARAM_TYPE, &type);
			}
			else if(!xmlStrcmp(buff, (const xmlChar *) "parallel")){
				type=BHDEV_TYPE_PARALLEL;
				dev.SetParameter(BHDEV_PARAM_TYPE, &type);
			}
			else if(!xmlStrcmp(buff, (const xmlChar *) "usb")){
				type=BHDEV_TYPE_USB;
				dev.SetParameter(BHDEV_PARAM_TYPE, &type);
			}
			else if(!xmlStrcmp(buff, (const xmlChar *) "network")){
				type=BHDEV_TYPE_NETWORK;
				dev.SetParameter(BHDEV_PARAM_TYPE, &type);
			}
			else if(!xmlStrcmp(buff, (const xmlChar *) "firewire")){
				type=BHDEV_TYPE_FIREWIRE;
				dev.SetParameter(BHDEV_PARAM_TYPE, &type);
			}
			else{
				error=3;
				return 3;
			}
			ParseOptions(node, dev);
		}
		node=node->next;
	}

	return 0;
}

void CDeviceParser::ParseOptions(xmlNodePtr node, CBhDevice& dev)
{
	xmlChar *buff;
	int	   temp;
	bool	  b;
	
	node=node->xmlChildrenNode;
	while(node!=NULL){
		if((!xmlStrcmp(node->name, (const xmlChar *) "manufacturer")) && (node->ns == ns)){
			buff=xmlGetProp(node, (const xmlChar *) "name");
			manufacturer=strdup((char *) buff);
		}
  	 if((!xmlStrcmp(node->name, (const xmlChar *) "model")) && (node->ns == ns)){
  	 	buff=xmlGetProp(node, (const xmlChar *) "name");
			model=strdup((char *) buff);
		}
		if((!xmlStrcmp(node->name, (const xmlChar *) "protocol")) && (node->ns == ns)){
			buff=xmlGetProp(node, (const xmlChar *) "type");
			protocol=strdup((char *) buff);
		}
		if((!xmlStrcmp(node->name, (const xmlChar *) "serial")) && (node->ns == ns)){
			buff=xmlGetProp(node, (const xmlChar *) "type");
		
			/*
			if(!xmlStrcmp(buff, (const xmlChar *) "default")){
				temp=BHDEV_SERIAL_TYPEDEFAULT;
				dev.SetParameter(BHDEV_PARAM_SERIAL, &temp);
			}
			else if(!xmlStrcmp(buff, (const xmlChar *) "terminal")){
				temp=BHDEV_SERIAL_TYPETERMINAL;
				dev.SetParameter(BHDEV_PARAM_SERIAL, &temp);
			}
			else if(!xmlStrcmp(buff, (const xmlChar *) "raw")){
				temp=BHDEV_SERIAL_TYPERAW;
				dev.SetParameter(BHDEV_PARAM_SERIAL, &temp);
			}
			
			buff=xmlGetProp(node, (const xmlChar *) "speed");
			
			buff=xmlGetProp(node, (const xmlChar *) "databits");
			
			buff=xmlGetProp(node, (const xmlChar *) "startbits");
			
			buff=xmlGetProp(node, (const xmlChar *) "stopbits");
			
			buff=xmlGetProp(node, (const xmlChar *) "parity");
			*/
		}
		if((!xmlStrcmp(node->name, (const xmlChar *) "parallel")) && (node->ns == ns)){
			buff=xmlGetProp(node, (const xmlChar *) "name");
			protocol=strdup((char *) buff);
		}
		
		node=node->next;
	}
}

int CDeviceParser::SaveDevice(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr node, CBhDevice& dev)
{
	return 0;
}

int CDeviceParser::GetError()
{
	return error;
}

char *CDeviceParser::GetErrorString(int err)
{
	switch(err){
	case 1: return "Namespace is null";
	case 2: return "Node is null";
	case 3: return "No valid attribute for device found";
	}
	return NULL;
}

}

}
