/*
datetime.cpp
Copyright (C)2003  Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.

*/

#include "datetime.h"
#include <time.h>

BhDateTime::BhDateTime()
{
   Now();
}

BhDateTime::BhDateTime(const BhDateTime& d) 
{ 
   jDay = d.jDay; 
   seconds=d.seconds;
}

BhDateTime& BhDateTime::operator=(const BhDateTime& d)
{
   jDay = d.jDay;
   seconds=d.seconds;
   return *this;
}

BhDateTime::BhDateTime(const BhTimestamp ts)
{
   jDay = tojd(ts.year, ts.month, ts.date);
   seconds=toseconds(ts.hour, ts.minute, ts.second);
}

BhDateTime::BhDateTime(const time_t t)
{
   struct tm *ptm;
   int y, m, d, h, mn, s;

   ptm = localtime(&t);
   y = ptm->tm_year + 1900;
   m = ptm->tm_mon + 1;
   d = ptm->tm_mday;
   h = ptm->tm_hour;
   mn =ptm->tm_min;
   s = ptm->tm_sec;
   
   jDay = tojd(y, m, d);
   seconds=toseconds(h, mn, s);
}

BhDateTime::BhDateTime(const char *s)
{
   jDay = atol(s);
}

BhDateTime::BhDateTime(BhTimestampFormat f, const char *s)
{
   int  x=2000;
   int  y=0;
   int  z=0;
   char buff[16];
   
   switch(f){
   case BHTSFORMAT_MYSQL_TS2:      // YY
      y+=atoi(s);
      jDay=tojd(x, 0, 0);
      seconds=0;
      break;
   case BHTSFORMAT_MYSQL_TS4:      // YYMM
      memcpy(buff, s, 2);
      buff[2]=0;
      y+=atoi(buff);
      memcpy(&buff[2], s, 2);
      buff[2]=0;
      x=atoi(buff);
      jDay=tojd(x, y, 0);
      seconds=0;
      break;
   case BHTSFORMAT_MYSQL_TS6:      // YYMMDD
      memcpy(&buff, s, 2);
      buff[2]=0;
      y+=atoi(buff);
      memcpy(&buff[2], s, 2);
      buff[2]=0;
      x=atoi(buff);
      memcpy(&buff[4], s, 2);
      buff[2]=0;
      z=atoi(buff);
      jDay=tojd(x, y, z);
      seconds=0;
      break;
   case BHTSFORMAT_MYSQL_TS8:      // YYYYMMDD
      memcpy(buff, s, 4);
      buff[4]=0;
      y=atoi(buff);
      memcpy(&buff[4], s, 2);
      buff[2]=0;
      x=atoi(buff);
      memcpy(&buff[6], s, 2);
      buff[2]=0;
      z=atoi(buff);
      jDay=tojd(x, y, z);
      seconds=0;
      break;
   case BHTSFORMAT_MYSQL_TS10:     // YYMMDDHHMM
      memcpy(buff, s, 2);
      buff[2]=0;
      y+=atoi(buff);
      memcpy(&buff[2], s, 2);
      buff[2]=0;
      x=atoi(buff);
      memcpy(&buff[4], s, 2);
      buff[2]=0;
      z=atoi(buff);
      jDay=tojd(x, y, z);
      memcpy(&buff[6], s, 2);
      buff[2]=0;
      x=atoi(buff);
      memcpy(&buff[8], s, 2);
      buff[2]=0;
      y=atoi(buff);
      seconds=toseconds(x, y, 0);
      break;
   case BHTSFORMAT_MYSQL_TS12:     // YYMMDDHHMMSS
      memcpy(buff, s, 2);
      buff[2]=0;
      y+=atoi(buff);
      memcpy(&buff[2], s, 2);
      buff[2]=0;
      x=atoi(buff);
      memcpy(&buff[4], s, 2);
      buff[2]=0;
      z=atoi(buff);
      jDay=tojd(x, y, z);
      memcpy(&buff[6], s, 2);
      buff[2]=0;
      x=atoi(buff);
      memcpy(&buff[8], s, 2);
      buff[2]=0;
      y=atoi(buff);
      memcpy(&buff[10], s, 2);
      buff[2]=0;
      z=atoi(buff);
      seconds=toseconds(x, y, z);
      break;
   case BHTSFORMAT_MYSQL_TS14:     // YYYYMMDDHHMMSS
      memcpy(buff, s, 4);
      buff[4]=0;
      y=atoi(buff);
      memcpy(&buff[4], s, 2);
      buff[2]=0;
      x=atoi(buff);
      memcpy(&buff[6], s, 2);
      buff[2]=0;
      z=atoi(buff);
      jDay=tojd(x, y, z);
      memcpy(&buff[8], s, 2);
      buff[2]=0;
      x=atoi(buff);
      memcpy(&buff[10], s, 2);
      buff[2]=0;
      y=atoi(buff);
      memcpy(&buff[12], s, 2);
      buff[2]=0;
      z=atoi(buff);
      seconds=toseconds(x, y, z);
      break;
   case BHTSFORMAT_MYSQL_TSDATE6:  //
   case BHTSFORMAT_MYSQL_TSDATE8:  //
   case BHTSFORMAT_MYSQL_TSTIME6:  //
   case BHTSFORMAT_MYSQL_TSTIME8:  //
      break;
   
   case BHTSFORMAT_MYSQL_TIME: //HH:MM:SS
      memcpy(buff, s, 2);
      buff[2]=0;
      y=atoi(buff);
      memcpy(&buff[3], s, 2);
      buff[2]=0;
      x=atoi(buff);
      memcpy(&buff[6], s, 2);
      buff[2]=0;
      z=atoi(buff);
      seconds=toseconds(x, y, z);
      break;
      
   case BHTSFORMAT_MYSQL_DATE: // YYYY-MM-DD
      memcpy(buff, s, 4);
      buff[4]=0;
      y=atoi(buff);
      memcpy(&buff[5], s, 2);
      buff[2]=0;
      x=atoi(buff);
      memcpy(&buff[8], s, 2);
      buff[2]=0;
      z=atoi(buff);
      jDay=tojd(x, y, z);
      break;
      
   // MySQL 4.1 - fixed size timestamp: 'YYYY-MM-DD HH:MM:SS'
   case BHTSFORMAT_MYSQL_TIMESTAMP: 
      memcpy(buff, s, 4);
      buff[4]=0;
      y=atoi(buff);
      memcpy(&buff[5], s, 2);
      buff[2]=0;
      x=atoi(buff);
      memcpy(&buff[8], s, 2);
      buff[2]=0;
      z=atoi(buff);
      jDay=tojd(x, y, z);
      memcpy(&buff[11], s, 2);
      buff[2]=0;
      x=atoi(buff);
      memcpy(&buff[14], s, 2);
      buff[2]=0;
      y=atoi(buff);
      memcpy(&buff[17], s, 2);
      buff[2]=0;
      z=atoi(buff);
      seconds=toseconds(x, y, z);
      break;
   }
}

BhDateTime::BhDateTime(const int day, const int month, const int year, const int hour, const int minute, const int second)
{
   jDay = tojd(year, month, day);
   seconds = toseconds(hour, minute, second);
}

void BhDateTime::Now()
{
   struct tm *ptm;
   time_t     t;
   
   int y, m, d, h, mn, s;

   t=time(NULL);
   ptm = localtime(&t);
   y = ptm->tm_year + 1900;
   m = ptm->tm_mon + 1;
   d = ptm->tm_mday;
   h = ptm->tm_hour;
   mn =ptm->tm_min;
   s = ptm->tm_sec;
   
   jDay = tojd(y, m, d);
   seconds=toseconds(h, mn, s);
}

int BhDateTime::Year()
{
   int y, m, d;

   fromjd(jDay, &y, &m, &d);
   
   return y;
}

int BhDateTime::Month()
{
   int y, m, d;

   fromjd(jDay, &y, &m, &d);
   
   return m;
}

int BhDateTime::Day()
{
   int y, m, d;

   fromjd(jDay, &y, &m, &d);
   
   return d;
}

int BhDateTime::DayOfWeek()
{
   return ((((int) (jDay % 7L)) + 1 ) % 7);
}

int BhDateTime::DayOfWorkWeek()
{
   return ((int) (jDay % 7L));
}

int BhDateTime::IsLeapYear()
{
   long jd1, jd2;
   int  y, m, d;
   
   fromjd(jDay, &y, &m, &d );
   
   jd1 = tojd(y, 2, 28);
   jd2 = tojd(y, 3, 1);
   return ((jd2 - jd1) > 1);
}

int BhDateTime::Hour()
{
   int h, m, s;

   fromseconds(seconds, &h, &m, &s);
   
   return h;
}

int BhDateTime::Minute()
{
   int h, m, s;

   fromseconds(seconds, &h, &m, &s);
   
   return m;
}

int BhDateTime::Second()
{
   int h, m, s;

   fromseconds(seconds, &h, &m, &s);
   
   return s;
}

void BhDateTime::ToBhTimeStamp(BhTimestamp *ts)
{
	ts->flags=0;
   ts->year=Year();
   ts->month=Month();
   ts->date=Day();
   ts->hour=Hour();
   ts->minute=Minute();
   ts->second=Second();
}

bool BhDateTime::operator==( const BhDateTime& dt) const
{
   if(jDay==dt.jDay && seconds==dt.seconds) return true;
   return false;
}

bool BhDateTime::operator!=( const BhDateTime& dt) const
{
   if(jDay!=dt.jDay || seconds!=dt.seconds) return true;
   return false;
}

bool BhDateTime::operator<( const BhDateTime& dt) const
{
   if(jDay < dt.jDay) return true;
   else if(jDay == dt.jDay){
      if(seconds < dt.seconds) return true;
      else return false;
   }
   return false;
}

bool BhDateTime::operator<=( const BhDateTime& dt) const
{
   if(jDay < dt.jDay) return true;
   else if(jDay == dt.jDay){
      if(seconds <= dt.seconds) return true;
      else return false;
   }
   return false;
}

bool BhDateTime::operator>( const BhDateTime& dt) const
{
   if(jDay > dt.jDay) return true;
   else if(jDay == dt.jDay){
      if(seconds > dt.seconds) return true;
      else return false;
   }
   return false;
}

bool BhDateTime::operator>=( const BhDateTime& d) const
{
   if(jDay > d.jDay) return true;
   else if(jDay == d.jDay){
      if(seconds >= d.seconds) return true;
      else return false;
   }
   return false;
}

BhDateTime& BhDateTime::operator+=(const BhDateTime& dt)
{
   jDay+=dt.jDay;
   return *this;
}
BhDateTime& BhDateTime::operator+=(const long d)
{
   jDay+=d;
   return *this;
}
BhDateTime& BhDateTime::operator-=(const BhDateTime& dt)
{
   jDay-=dt.jDay;
   return *this;
}
BhDateTime& BhDateTime::operator-=(const long d)
{
   jDay-=d;
   return *this;
}
BhDateTime& BhDateTime::operator++()
{
   jDay++;
   return *this;
}
BhDateTime& BhDateTime::operator--()
{
   jDay--;
   return *this;
}

int BhDateTime::toseconds(const int h, const int m, const int s)
{
   seconds=s + (m *60) + (h * 24 *60);
   return seconds;
}    

void BhDateTime::fromseconds(const int sec, int *h, int *m, int *s)
{
   int th, tm, ts;
   
   th = (sec / 3600);
   tm = sec - (th * 60);
   ts = sec - (th * 60) - (tm * 60);
   
   *h=th;
   *m=tm;
   *s=ts;
} 
    
long BhDateTime::tojd(const int y, const int m, const int d)
{
   long jul_day;
   int a,b;
   int year = y, month = m, day = d;
   float year_corr;

   if(year < 0)
      year++;
   year_corr = ( year > 0 ? 0.0 : 0.75 );
   if(month <= 2 ){
      year--;
      month += 12;
   }
   b = 0;
   if(year * 10000.0 + month * 100.0 + day >= 15821015.0){
      a = year / 100;
      b = 2 - a + a / 4;
   }
   jul_day=(long) (365.25 * year - year_corr) + (long) (30.6001 * (month + 1)) + day + 1720995L + b;

   return jul_day;
}

void BhDateTime::fromjd(const long jd, int *y, int *m, int *day)
{
   long a, b, c, d, e, z, alpha;

   z=jd;
   if(z < 2299161L)
      a=z;
   else{
       alpha=(long) ((z - 1867216.25) / 36524.25 );
       a=z + 1 + alpha - alpha / 4;
   }
   b = a + 1524;
   c = (long) (( b - 122.1) / 365.25);
   d = (long) (365.25 * c);
   e = (long) ((b - d) / 30.6001);
   *day= (int) b - d - (long) (30.6001 * e);
   *m= (int) (e < 13.5) ? e - 1 : e - 13;
   *y= (int) (*m > 2.5) ? (c - 4716) : c - 4715;
   if(*y <= 0)
      *y -= 1;
}

void BhDateTime::ToString(char *szBuffer)
{
   int i;
   long Temp;

   Temp=jDay;
   if(Temp < 0L )
      szBuffer[0] = '-';
   else
      szBuffer[0] = '+';
   szBuffer[11] = '\0';
   for(i=10; i > 0; i--){
      szBuffer[i]=(Temp % 10L) + '0';
      Temp/=10;
   }
}

int BhDateTime::DayOfYear()
{
   int y, m, d;
   long soy;

   fromjd( jDay, &y, &m, &d );
   soy = tojd( y, 1, 1 );
   return (int) (jDay - soy + 1);
}

void BhDateTime::ToSysTime(struct tm *t)
{
   int y, m, d;
    
   fromjd(jDay, &y, &m, &d);
   if(y < 1970){
      y = 70;
      m = 1;
      d = 1;
   }
   t->tm_year = y - 1900 ;
   t->tm_mon = m-1;
   t->tm_mday = d;
   t->tm_hour = Hour();
   t->tm_min = Minute();
   t->tm_sec = Second();
   t->tm_isdst = 0;
}
