/*
datetime.h
Copyright (C)2003 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY
*/
#ifndef _BHDATETIME_H_
#define _BHDATETIME_H_

#include <cstring>
#include <cstdlib>
#include <ctime>

#ifndef _BHHDR_H_
#include "pos.h"
#endif

/**
 * @defgroup dt Date/Time
 * @{
 *
 * @ingroup commlib
 * @{	
 */

enum BhTimestampFormat{
   BHTSFORMAT_MYSQL_TIME,
   BHTSFORMAT_MYSQL_DATE,
   BHTSFORMAT_MYSQL_TIMESTAMP,   //!MySQL 4.1 compliant stamp
   BHTSFORMAT_MYSQL_TS2,     /*!Time stamp format - MySQL timestamp 2*/
   BHTSFORMAT_MYSQL_TS4,     /*!Time stamp format - MySQL timestamp 4*/
   BHTSFORMAT_MYSQL_TS6,     /*!Time stamp format - MySQL timestamp 6*/
   BHTSFORMAT_MYSQL_TS8,     /*!Time stamp format - MySQL timestamp 8*/
   BHTSFORMAT_MYSQL_TS10,    /*!Time stamp format - MySQL timestamp 10*/
   BHTSFORMAT_MYSQL_TS12,    /*!Time stamp format - MySQL timestamp 12*/
   BHTSFORMAT_MYSQL_TS14,    /*!Time stamp format - MySQL timestamp 14*/
   BHTSFORMAT_MYSQL_TSDATE6, /*!Time stamp format - MySQL date 6*/
   BHTSFORMAT_MYSQL_TSDATE8, /*!Time stamp format - MySQL date 8*/
   BHTSFORMAT_MYSQL_TSTIME6, /*!Time stamp format - MySQL time 6*/
   BHTSFORMAT_MYSQL_TSTIME8  /*!Time stamp format - MySQL time 8*/
};

class BhDateTime
{
private:
   long jDay;
   int  seconds;
   
   long tojd(const int y, const int m, const int d);
   void fromjd(const long jd, int *y, int *m, int *d);
   int  toseconds(const int h, const int m, const int s);
   void fromseconds(const int sec, int *h, int *m, int *s);

public:
   BhDateTime();
   BhDateTime(const BhDateTime& ts);
   BhDateTime& operator=(const BhDateTime& ts);
   BhDateTime(const time_t t);
   BhDateTime(const char *s);
   BhDateTime(const BhTimestamp ts);
   BhDateTime(BhTimestampFormat f, const char *s);
   BhDateTime(const int day, const int month, const int year, const int hour, const int minute, const int second);

   void Now();
   /*!
		Year
		\return The year.
	*/
   int Year();
   /*!
		Month
		\return The month.
	*/
   int Month();
   /*!
		Day
		\return The day.
	*/
   int Day();
   /*!
		DayOfYear
		\return The day of the year.
	*/
   int DayOfYear();
   int DayOfWeek();
   int DayOfWorkWeek();
   int IsLeapYear();
   int Hour();
   int Minute();
   int Second();
	
	/*! formatting operations */
	void ToBhTimeStamp(BhTimestamp *);

   /*! numerical operations */
   BhDateTime& operator+=(const BhDateTime& dt);
   BhDateTime& operator+=(const long d);
   BhDateTime& operator-=(const BhDateTime& dt);
   BhDateTime& operator-=(const long d);
   BhDateTime& operator++();
   BhDateTime& operator--();
   
   /*! comparison operations */
   bool operator==(const BhDateTime& dt) const;
   bool operator!=(const BhDateTime& dt) const;
   bool operator<(const BhDateTime& dt) const;
   bool operator<=(const BhDateTime& dt) const;
   bool operator>(const BhDateTime& dt) const;
   bool operator>=(const BhDateTime& dt) const;
   
   void ToString(char *);
   /*!
		ToSysTime
		\param t A tm struct to be filled.
	*/
   void ToSysTime(struct tm *t);
};

/* @} 
	@} */

#endif
