/*
grp.cpp 
Copyright (C)2001  Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.

*/
#include <cstdio>
#include <cstring>

#include "bhgrp.h"
	
namespace bhpos{
   
CStation::CStation()
{
	ID=0;
	name=NULL;
	sitename=NULL;
	deptname=NULL;
	groupname=NULL;
}

CStation::CStation(const CStation& s)
{
   ID=0;
	name=NULL;
	sitename=NULL;
	deptname=NULL;
	groupname=NULL;
	
	ID=s.ID;
	
	if(s.sitename) SetSite(s.siteID, s.sitename);
	else SetSite(s.siteID, NULL);
	
	if(s.deptname) SetDept(s.deptID, s.deptname);
	else SetDept(s.deptID, NULL);
	
	if(s.groupname) SetSite(s.groupID, s.groupname);
	else SetGroup(s.groupID, NULL);
	
	if(s.name) SetName(s.name);
}

CStation::CStation(char *n, int i)
{
	SetName(n);
	SetID(i);
	sitename=NULL;
	deptname=NULL;
	groupname=NULL;
}

CStation::~CStation()
{
	if(name) delete name;
	if(sitename) delete sitename;
	if(deptname) delete  deptname;
	if(groupname) delete groupname;
}
CStation &CStation::operator=(CStation const& s)
{
	this->ID=s.ID;
	if(s.sitename) this->SetSite(s.siteID, s.sitename);
	else this->SetSite(s.siteID, NULL);
	
	if(s.deptname) this->SetDept(s.deptID, s.deptname);
	else this->SetDept(s.deptID, NULL);
	
	if(s.groupname) this->SetSite(s.groupID, s.groupname);
	else this->SetGroup(s.groupID, NULL);
	
	if(s.name) this->SetName(s.name);
	
	return *this;
}

bool CStation::operator==(CStation const& s)
{
	if(ID==s.ID && groupID==s.groupID &&
		deptID==s.deptID && siteID==s.siteID) return true;
	return false;
}
bool CStation::operator<(CStation const& s)
{
	if(ID < s.ID && groupID==s.groupID &&
		deptID==s.deptID && siteID==siteID) return true;
	return false;
}

char *CStation::GetName()
{
	return name;
}

void CStation::SetName(char *n)
{
   if(name) delete name;
	if(!name) name=new char[strlen(n) + 1];
	if(name) strcpy(name, n);
}

int CStation::GetID()
{
	return ID;
}
void CStation::SetID(int i) 
{
	ID=i;
}

void CStation::GetEnteredAt(char *n)
{
   if(n==NULL) return;
   
   sprintf(n, "%05d%05d%05d%05d", siteID, deptID, groupID, ID);
}

void CStation::SetGroup(int i, char *n)
{
   if(groupname) delete groupname;
   
	if(!groupname && n){
		groupname=new char[strlen(n) + 1];
		if(groupname) strcpy(groupname, n);
	}
	groupID=i;
}
void CStation::SetDept(int i, char *n)
{
   if(deptname) delete deptname;
	if(!deptname && n){
		deptname=new char[strlen(n) + 1];
		if(deptname) strcpy(deptname, n);
	}
	deptID=i;
}
void CStation::SetSite(int i, char *n)
{
   if(sitename) delete sitename;
   if(n){
		sitename=new char[strlen(n) + 1];
		if(sitename) strcpy(sitename, n);
	}
	siteID=i;
}
void CStation::GetGroup(int *i, char *n)
{
	*i=groupID;
	if(groupname && n) strcpy(n, groupname);
}
void CStation::GetDept(int *i, char *n)
{
	*i=deptID;
	if(deptname && n) strcpy(n, deptname);
}
void CStation::GetSite(int *i, char *n)
{
	*i=siteID;
	if(sitename && n) strcpy(n, sitename);
}

////////////////////////////////////////////////////////////////
//
//
//
//
////////////////////////////////////////////////////////////////
	
CGroup::CGroup()
{
	ID=0;
	name=NULL;
}

CGroup::CGroup(const CGroup& g)
{
	ID=g.ID;
	if(g.name) SetName(g.name);
	stations=g.stations;
}

CGroup::CGroup(char *n, int i)
{
	SetName(n);
	SetID(i);
}

CGroup::~CGroup()
{
	if(name) delete name;
	stations.clear();
}

CGroup &CGroup::operator=(CGroup const& g)
{
	this->ID=g.ID;
	if(g.name) this->SetName(g.name);
	this->stations=g.stations;
	
	return *this;
}

bool CGroup::operator==(CGroup const& g)
{
	if(ID==g.ID) return true;
	return false;
}
bool CGroup::operator<(CGroup const& g)
{
	if(ID < g.ID) return true;
	return false;
}


char *CGroup::GetName()
{
	return name;
}

void CGroup::SetName(char *n)
{
   if(name) delete name;
	if(!name) name=new char[strlen(n) + 1];
	if(name) strcpy(name, n);
}

int CGroup::GetID()
{
	return ID;
}
void CGroup::SetID(int i) 
{
	ID=i;
}

int CGroup::AddStation(CStation& s)
{
	stations.push_back(s);
	return 0;
}

int CGroup::NumStations()
{
	return stations.size();
}

int CGroup::GetStation(int i, CStation& s)
{
	std::list<CStation>::iterator iter=stations.begin();
	int temp=i;
	
	while(temp){ 
		iter++;
		temp--;
	}
	s=*iter;
	
	return 0;
}

////////////////////////////////////////////////////////////////
//
//
//
//
////////////////////////////////////////////////////////////////
	

CDepartment::CDepartment()
{
	ID=0;
	name=NULL;
}

CDepartment::CDepartment(const CDepartment& d)
{
	ID=d.ID;
	if(d.name) SetName(d.name);
	groups=d.groups;
}

CDepartment::CDepartment(char *n, int i)
{
	SetName(n);
	SetID(i);
}

CDepartment::~CDepartment()
{
	if(name) delete [] name;
	groups.clear();
}

CDepartment &CDepartment::operator=(const CDepartment& d)
{
	this->ID=d.ID;
	if(d.name) this->SetName(d.name);
	this->groups=d.groups;
	
	return *this;
}
bool CDepartment::operator==(const CDepartment& d)
{
	if(ID==d.ID) return true;
	return false;
}
bool CDepartment::operator<(const CDepartment& d)
{
	if(ID < d.ID) return true;
	return false;
}

char *CDepartment::GetName()
{
	return name;
}

void CDepartment::SetName(char *n)
{
   if(name) delete name;
	if(!name) name=new char[strlen(n) + 1];
	if(name) strcpy(name, n);
}

int CDepartment::GetID()
{
	return ID;
}
void CDepartment::SetID(int i) 
{
	ID=i;
}

int CDepartment::AddGroup(CGroup& g)
{
	groups.push_back(g);
	return 0;
}

int CDepartment::NumGroups()
{
	return groups.size();
}

int CDepartment::GetGroup(int i, CGroup& g)
{
	std::list<CGroup>::iterator iter=groups.begin();
	int temp=i;
	
	while(temp){ 
		iter++;
		temp--;
	}
	g=*iter;
	
	return 0;
}


////////////////////////////////////////////////////////////////
//
//
//
//
////////////////////////////////////////////////////////////////

CSite::CSite()
{
	ID=0;
	name=NULL;
}

CSite::CSite(const CSite& s)
{
	ID=s.ID;
	if(s.name) SetName(s.name);
}

CSite::CSite(char *n, int i)
{
	SetName(n);
	SetID(i);
}

CSite::~CSite()
{
	if(name) delete [] name;
	departments.clear();
}

CSite &CSite::operator=(const CSite& s)
{
	this->ID=s.ID;
	if(s.name) this->SetName(s.name);
	
	return *this;
}

bool CSite::operator==(const CSite& s)
{
	if(ID==s.ID) return true;
	return false;
}

bool CSite::operator<(const CSite& s)
{
	if(ID<s.ID) return true;
	return false;
}

char *CSite::GetName()
{
	return name;
}

void CSite::SetName(char *n)
{
   if(name) delete name;
	if(!name) name=new char[strlen(n) + 1];
	if(name) strcpy(name, n);
}
int CSite::GetID()
{
	return ID;
}
void CSite::SetID(int i) 
{
	ID=i;
}

int CSite::AddDeparment(CDepartment& d)
{
	departments.push_back(d);
	return 0;
}

int CSite::NumDepartments()
{
	return departments.size();
}

int CSite::GetDepartment(int i, CDepartment& d)
{
	std::list<CDepartment>::iterator iter=departments.begin();
	int temp=i;
	
	while(temp){ 
		iter++;
		temp--;
	}
	d=*iter;
	
	return 0;
}

////////////////////////////////////////////////////////////////
//
//
//
//
////////////////////////////////////////////////////////////////

COrganization::COrganization()
{
	ID=0;
	name=NULL;
}

COrganization::COrganization(const COrganization& s)
{
	ID=s.ID;
	if(s.name) SetName(s.name);
}

COrganization::COrganization(char *n, int i)
{
	SetName(n);
	SetID(i);
}

COrganization::~COrganization()
{
	if(name) delete [] name;
	sites.clear();
}

COrganization &COrganization::operator=(const COrganization& s)
{
	this->ID=s.ID;
	if(s.name) this->SetName(s.name);
	
	return *this;
}

bool COrganization::operator==(const COrganization& s)
{
	if(ID==s.ID) return true;
	return false;
}

bool COrganization::operator<(const COrganization& s)
{
	if(ID<s.ID) return true;
	return false;
}

char *COrganization::GetName()
{
	return name;
}

void COrganization::SetName(char *n)
{
   if(name) delete name;
	if(!name) name=new char[strlen(n) + 1];
	if(name) strcpy(name, n);
}
int COrganization::GetID()
{
	return ID;
}
void COrganization::SetID(int i) 
{
	ID=i;
}

int COrganization::AddSite(CSite& d)
{
	sites.push_back(d);
	return 0;
}

int COrganization::NumSites()
{
	return sites.size();
}

int COrganization::GetSite(int i, CSite& d)
{
	std::list<CSite>::iterator iter=sites.begin();
	int temp=i;
	
	while(temp){ 
		iter++;
		temp--;
	}
	d=*iter;
	
	return 0;
}

}
