/*
grp.h - simple class to ease the site/dept/group/station heirarchy
Copyright (C)2001  Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.

*/
#ifndef _BHGRP_H_
#define _BHGRP_H_

#include <list>

namespace bhpos{

/**
 * @defgroup grp Groups
 * @{
 *
 * \ingroup commlib
 * @{	
 */

class CStation
{
private:
	int	ID;
	char *name;
	
	int   groupID;
	char *groupname;
	int   deptID;
	char *deptname;
	int   siteID;
	char *sitename;
	
public:
	CStation();
	CStation(CStation const&);
	CStation(char *, int);
	~CStation();
	CStation &operator=(CStation const&);
	bool operator==(CStation const&);
	bool operator<(CStation const&);
	char *GetName();
	void SetName(char *);
	int  GetID();
	void SetID(int);
	void SetGroup(int, char *);
	void SetDept(int, char *);
	void SetSite(int, char *);
	void GetGroup(int *, char *);
	void GetDept(int *, char *);
	void GetSite(int *, char *);
	void GetEnteredAt(char *);
};

/* @}
 * \ingroup grp
 * @{
 */

class CGroup
{
private:
	int	ID;
	char *name;
	std::list<CStation> stations;
	
public:
	CGroup();
	CGroup(CGroup const&);
	CGroup(char *, int);
	~CGroup();
	CGroup &operator=(CGroup const&);
	bool operator==(CGroup const&);
	bool operator<(CGroup const&);
	
	char *GetName();
	void SetName(char *);
	int GetID();
	void SetID(int);
	int AddStation(CStation&);
	int NumStations();
	int GetStation(int i, CStation&);
};

/* @}
 * \ingroup grp
 * @{
 */

class CDepartment
{
private:
	int	ID;
	char	*name;
	std::list<CGroup> groups;
	
public:
	CDepartment();
	CDepartment(CDepartment const&);
	CDepartment(char *, int);
	~CDepartment();
	CDepartment &operator=(CDepartment const&);
	bool operator==(CDepartment const&);
	bool operator<(CDepartment const&);
	
	char *GetName();
	void SetName(char *);
	int GetID();
	void SetID(int);
	int AddGroup(CGroup&);
	int NumGroups();
	int GetGroup(int i, CGroup&);
};

/* @}
 * \ingroup grp
 * @{
 */

class CSite
{
private:
	int	ID;
	char *name;
	std::list<CDepartment> departments;
	
public:
	CSite();
	CSite(CSite const&);
	CSite(char *, int);
	~CSite();
	CSite &operator=(CSite const&);
	bool operator==(CSite const&);
	bool operator<(CSite const&);
	
	char *GetName();
	void  SetName(char *);
	int   GetID();
	void  SetID(int);
	int   AddDeparment(CDepartment&);
	int   NumDepartments();
	int   GetDepartment(int i, CDepartment&);
	
};

/* @}
 * \ingroup grp
 * @{
 */

class COrganization
{
private:
	int	ID;
	char *name;
	std::list<CSite> sites;
	
public:
	COrganization();
	COrganization(COrganization const&);
	COrganization(char *, int);
	~COrganization();
	COrganization &operator=(COrganization const&);
	bool operator==(COrganization const&);
	bool operator<(COrganization const&);
	
	char *GetName();
	void  SetName(char *);
	int   GetID();
	void  SetID(int);
	int   AddSite(CSite&);
	int   NumSites();
	int   GetSite(int i, CSite&);
	
};

/* @} 
	@} */

}

#endif
