/*
locale.cpp - a locale class
Copyright (C) 2001  Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY, or INTERESTING SANDSTONE SAMPLES. 
	The author takes no responsibility for anything at all, even god existing.
    
*/
#include <pthread.h>
#include <cstdio>
#include <ctime>
#include <cstring>
#include <cmath>
#include <malloc.h>

#include "clocale.h"

pthread_mutex_t localemutex=PTHREAD_MUTEX_INITIALIZER;

namespace bhpos{

CLocale::CLocale()
{
   cat=LC_ALL;
	lc=NULL;
	old_locale=NULL;
	buff[0]='\0';
}

CLocale::~CLocale()
{
	ResetLocale();
	if(lc) free(lc);
	pthread_mutex_destroy(&localemutex);
}

int CLocale::SetLocale(int category, char *locale)
{
	pthread_mutex_lock(&localemutex);
	cat=category;
	char *boo=setlocale(category, NULL);
	old_locale=strdup(boo);
	
	setlocale(category, locale);

	lc=(lconv *) malloc(sizeof(lconv));
	if(!lc){
		return -1;
	}
	lc=localeconv();
	
	pthread_mutex_unlock(&localemutex);
	
	return 0;
}

void CLocale::ResetLocale()
{
	pthread_mutex_lock(&localemutex);
	setlocale(cat, old_locale);
	free(old_locale);
	pthread_mutex_unlock(&localemutex);
}

char *CLocale::FormatCurrency(const double c)
{
	char temp[64];
	double i, x, f, t, y;
	int	 z;
	
	pthread_mutex_lock(&localemutex);
	
	if(lc->p_cs_precedes){
		strcpy(buff, lc->int_curr_symbol);
		if(lc->p_sep_by_space){
			strcat(buff, " ");
		}
	}

   f=modf(c, &i);

	if(c > 1000.0){	// need seperator
		t=c;
		while(t > 1000.0){
			t/=1000;
			y=modf(t, &x);

			sprintf(temp, "%i%c", x, lc->mon_thousands_sep);
			strcat(buff, temp);
		}
		sprintf(temp, "%c%i", lc->mon_decimal_point, f);
		strcat(buff, temp);
	}
	else{
		sprintf(temp, "%i%c%i", i, lc->mon_decimal_point, f);
		strcat(buff, temp);
	}

	if(!lc->p_cs_precedes){
		if(lc->p_sep_by_space){
			strcat(buff, " ");
		}
		strcat(buff, lc->int_curr_symbol);
	}
	
	pthread_mutex_unlock(&localemutex);

	return buff;
}

char *CLocale::FormatTime(const tm *time)
{
	size_t s;
	
	pthread_mutex_lock(&localemutex);
	
	s=strftime(buff, 64, "%X", time);
	if(s==0) return NULL;
	
	pthread_mutex_unlock(&localemutex);
	
	return buff;
}

char *CLocale::FormatDate(const tm *time)
{
	size_t s;
	
	pthread_mutex_lock(&localemutex);
	
	s=strftime(buff, 64, "%x", time);
	if(s==0) return NULL;

	pthread_mutex_unlock(&localemutex);	
	
	return buff;
}

}
