/*
cpayitem.cpp - a pay item shared library.

Copyright (C) 2001 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2004 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 
*/


#include "ctransitems.h"

namespace bhpos{

CPayItem::CPayItem()
{
	ID=PTYPE_UNKNOWN;
	flags=PTYPE_FLAG_NONE;
	name="";
	amount=0.0;
	//memset(&timestamp, 0, sizeof(BhDateTime ));
	site=0;
   dept=0;
   group=0;
   station=0;
   user=0;
	auth=0;
}
CPayItem::CPayItem(const CPayItem& pi)
{
	ID=pi.ID;
	flags=pi.flags;
	amount=pi.amount;
	name=pi.name;
	timestamp=pi.timestamp;
	site=pi.site;
   dept=pi.dept;
   group=pi.group;
   station=pi.station;
   user=pi.user;
	auth=pi.auth;
}

CPayItem::CPayItem(BhPayType i, char *s, double f)
{
	ID=i;
	flags=PTYPE_FLAG_NONE;
	amount=f;
	name=s;
	//memset(&timestamp, 0, sizeof(BhDateTime ));
	site=0;
   dept=0;
   group=0;
   station=0;
   user=0;
	auth=0;
}

CPayItem::~CPayItem()
{
}

CPayItem &CPayItem::operator=(const CPayItem& pi)
{
	this->ID=pi.ID;
	this->flags=pi.flags;
	this->amount=pi.amount;
	name=pi.name;
	this->timestamp=pi.timestamp;
	this->site=pi.site;
   this->dept=pi.dept;
   this->group=pi.group;
   this->station=pi.station;
   this->user=pi.user;
	this->auth=pi.auth;
	return *this;
}

CPayItem* CPayItem::operator&()
{
   return this;
}

bool CPayItem::operator==(const CPayItem &p)
{
	if(ID==p.ID) return true;
	return false;
}

bool CPayItem::operator!=(const CPayItem& p)
{
	if(ID!=p.ID) return true;
	return false;
}

bool CPayItem::operator<(const CPayItem& p)
{
	if(amount<p.amount) return true;
	return false;
}

void CPayItem::Clear()
{
	ID=PTYPE_UNKNOWN;
	flags=PTYPE_FLAG_NONE;
	amount=0.0;
   name="";
	memset(&timestamp, 0, sizeof(BhDateTime ));
	site=0;
   dept=0;
   group=0;
   station=0;
   user=0;
	auth=0;
}

void CPayItem::SetAmount(double f) 
{
	amount=f;
}

void CPayItem::AddAmount(double f) 
{
	amount+=f;
}

const double CPayItem::GetAmount()
{
	return amount;
}

const BhPayType CPayItem::GetID()
{
	return ID;
}

void CPayItem::SetID(BhPayType i)
{
	ID=i;
}

const BhPayTypeFlag CPayItem::GetFlags()
{
	return flags;
}

void CPayItem::SetFlags(BhPayTypeFlag i)
{
	flags=i;
}

bool CPayItem::SetName(char *s)
{
	name=s;
	return true;
}

const char *CPayItem::GetName()
{
	return name.data();
}
void CPayItem::SetWhere(char *w)
{
   if(w==NULL) return;
   
   char temp[6];
   
   memcpy(temp, w, 5);
   temp[6]=0;
   site=atoi(temp);
   memcpy(temp, &w[5], 5);
   temp[6]=0;
   dept=atoi(temp);
   memcpy(temp, &w[10], 5);
   temp[6]=0;
   group=atoi(temp);
   memcpy(temp, &w[15], 5);
   temp[6]=0;
   station=atoi(temp);
}
void CPayItem::SetTimestamp(BhDateTime& dt)
{
   timestamp=dt;
}

void CPayItem::GetTimestamp(BhTimestamp *ts)
{
   ts->flags=14;
   ts->year=timestamp.Year();
   ts->month=timestamp.Month();
   ts->date=timestamp.Day();
   ts->hour=timestamp.Hour();
   ts->minute=timestamp.Minute();
   ts->second=timestamp.Second();
}

const BhDateTime& CPayItem::GetTimestamp()
{
   return timestamp;
}

}
