/*
ctransitems.h
Copyright (C)2001 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 

   Part of:
   BhPos, Copyright (C) 2001-2005 Al Kingsley
   BhPos comes with ABSOLUTELY NO WARRANTY.

*/
#ifndef _CTRANSITEM_
#define _CTRANSITEM_

#include <glibmm/ustring.h>
//#include <string>
#include <list>
#include <algorithm>

#ifndef _BHHDR_H_
#include "pos.h"
#endif

#ifndef _BHDATETIME_H_
#include "datetime.h"
#endif 

namespace bhpos{
	
/**
 * @defgroup trans Transactions
 * @{
 *
 * @ingroup commlib
 * @{	
 */

   /*! BhPluType
 \brief The plu type.
*/
enum BhPluType{
   BHSI_PLUTYPE_UNKNOWN,
   BHSI_PLUTYPE_EAN13,
   BHSI_PLUTYPE_UPCA,
   BHSI_PLUTYPE_ISBN,
   BHSI_PLUTYPE_MFGPROD
};

/*! BhSaleItemType
 \brief The sale item type.
*/
enum BhSaleItemType{
   BHSI_SITYPE_UPLU,
   BHSI_SITYPE_PLU,         //! - standard sale item - a standard barcode
   BHSI_SITYPE_PRICE,       //! - unknown PLU with price
   BHSI_SITYPE_DESCPRICE,   //! - unknown plu with desc/price
   BHSI_SITYPE_ALL,         //! - unknown plu with full entry
   BHSI_SITYPE_RETURN,      //! - return in sale
	BHSI_SITYPE_LAYBY,       //! - layby in sale
   BHSI_SITYPE_DETAIL,      //! - detailed
	BHSI_SITYPE_DELIVERY,    //! - delivered item
	BHSI_SITYPE_UNDELIVERED, //! - undelivered item
	BHSI_SITYPE_RENTAL,      //! - rental item
	BHSI_SITYPE_RENTALRETURN,//! - rental return item
	BHSI_SITYPE_EPHEMERAL    //! - ephemeral item
};

/*! BhItemDiscountType
 \brief The discount type.
*/
enum BhItemDiscountType{
   BHSI_DISCOUNT_NONE=0,
   BHSI_DISCOUNT_USER,
   BHSI_DISCOUNT_PROMO,
   BHSI_DISCOUNT_PRICEBREAK,
   BHSI_DISCOUNT_CUSTOMER,
   BHSI_DISCOUNT_USERTRANS,
   BHSI_DISCOUNT_CUSTOMERTRANS,
   BHSI_DISCOUNT_PROMOTRANS
};

enum BhItemDepositType{
   BHSI_DEPOSIT_NONE=0,
   BHSI_DEPOSIT_REFUNDABLE,    //! refundable deposit
   BHSI_DEPOSIT_REDEMPTION     //! redemption deposit
};

/*! CSaleItem
 \brief A class to handle sales items
*/
class CSaleItem{
private:
   Glib::ustring  PLU;
   unsigned char  plutype;
   unsigned char  type;
   Glib::ustring  displayText;
   Glib::ustring  recuText;
   int            quantity;
   float          tax;
   float          discount;
   float          unitPrice;
   float          deposit;
   int            reason;
   unsigned char  flags;
   Glib::ustring  flag_msg;
   Glib::ustring  deposit_name;

   BhItemDiscountType discount_type;
   BhItemDepositType  deposit_type;

protected:
   unsigned short site;
   unsigned short dept;
   unsigned short group;
   unsigned short station;
   int            user;
   BhDateTime     timestamp;

public:
   CSaleItem();
   CSaleItem(const CSaleItem&);
   CSaleItem(std::string&);
   CSaleItem(Glib::ustring&);
   ~CSaleItem();
   CSaleItem& operator=(const CSaleItem&);
   //CSaleItem* operator&();
   bool operator==(const CSaleItem&);
   bool operator!=(const CSaleItem&);
   bool operator<(const CSaleItem&);
   
   //! Clear to default values
   void  Clear();
   
   const int            GetType();
   const int            PluSize();
   const unsigned char  GetPluType();
   const Glib::ustring  GetPLU();
   const Glib::ustring  GetDispText();
   const Glib::ustring  GetRecText();
   const int            GetQuantity();
   const float          GetTax();
   const float          GetDiscount();
   const float          GetTotalDiscount();
   BhItemDiscountType   GetDiscountType(){ return discount_type; }
   BhItemDepositType    GetDepositType(){ return deposit_type; }
   const Glib::ustring  GetDepositName();
   const float          GetPrice();
   const float          GetSubTotal();
   const float          GetTotal();
   const float          GetTotal(bool bTaxInclusive);
   const float          GetDeposit();
   const unsigned char  GetFlags(){ return flags; }
   const Glib::ustring  GetMsg();
   const int            GetReason(){ return reason; }
   const BhDateTime&    GetTimestamp();
   void                 GetTimestamp(BhTimestamp *);
   const int            GetUser(){ return user; }
   const unsigned short GetSite(){ return site; }
   const unsigned short GetDept(){ return dept; }
   const unsigned short GetGroup(){ return group; }
   const unsigned short GetStation(){ return station; }
   
   void  SetType(int);
   void  SetPluType(unsigned char);
   bool  SetPLU(const char *);
   bool  SetPLU(Glib::ustring);
   bool  SetDispText(char *);
   bool  SetDispText(Glib::ustring);
   bool  SetRecText(char *);
   bool  SetRecText(Glib::ustring);
   bool  SetQuantity(int);
   bool  SetTax(float);
   bool  SetDiscount(float);
   void  SetDiscountType(BhItemDiscountType t){ discount_type=t; }
   void  SetDepositType(BhItemDepositType t){ deposit_type=t; }
   void  SetDepositName(char *deposit_name);
   void  SetDepositName(Glib::ustring);
   bool  SetPrice(float);
   void  SetDeposit(float);
   void  SetFlags(unsigned char f){ flags=f; }
   void  SetMsg(char *);
   void  SetMsg(Glib::ustring);
   void  SetReason(int i){ reason=i; }
   void  SetTimestamp(BhDateTime&);
   void  SetUser(int i) { user=i; }
   void  SetWhere(char *);
   void  SetSite(unsigned short i) { site=i; }
   void  SetDept(unsigned short i) { dept=i; }
   void  SetGroup(unsigned short i) { group=i; }
   void  SetStation(unsigned short i) { station=i; }
};

enum BhPayType{
   PTYPE_UNKNOWN=0,     //! - unknown
   PTYPE_CASH,          //! - cash
   PTYPE_CREDIT,        //! - credit
   PTYPE_CHEQUE,        //! - cheque
   PTYPE_SMARTC,        //! - smart card
   PTYPE_CREDITRETURN,  //! - credit return
   PTYPE_CREDITSLIP,    //! - credit slip
   PTYPE_GIFTVOUCHER,   //! - gift voucher
   PTYPE_TPVOUCHER,     //! - third party voucher
   PTYPE_INTCOUPON,     //! - internal coupon
   PTYPE_TPCOUPON,      //! - third party coupon
   PTYPE_LUNCHSTAMP,    //! - luncheon voucher
   PTYPE_INHOUSECHARGE, //! - in-house charge account
   PTYPE_STORECARD,     //! - store card
   PTYPE_OTHER,         //! - other
   PTYPE_CREDIT_CB,     //! - credit cash back
   PTYPE_CHEQUE_CB,     //! - cheque cash back
   PTYPE_SMARTC_CB,     //! - smart card cash back
   PTYPE_TOKEN_CB,
   PTYPE_VOUCHER_CB,
   PTYPE_COUPON_CB,
   PTYPE_OTHER_CB
};

enum BhPayTypeFlag{
   PTYPE_FLAG_NONE=0,
   PTYPE_FLAG_SIMPLEAUTH,
   PTYPE_FLAG_PREAUTH,
   PTYPE_FLAG_DONE,
   PTYPE_FLAG_FAIL,
   PTYPE_FLAG_ERROR,
   PTYPE_FLAG_CREDITSIEZE
};

/*! CPayItem
 \brief A class to handle pay items
*/
class CPayItem{
private:
   BhPayType     ID;
   BhPayTypeFlag flags;
   std::string   name;
   double        amount;

protected:
   BhDateTime      timestamp;
   unsigned short  site;
   unsigned short  dept;
   unsigned short  group;
   unsigned short  station;
   int             user;
	long				 auth;

public:
   CPayItem();
   CPayItem(const CPayItem&);
   CPayItem(BhPayType, char *, double);
   ~CPayItem();
   
   CPayItem &operator=(const CPayItem&);
   CPayItem* operator&();
   bool operator==(const CPayItem&);
   bool operator!=(const CPayItem&);
   bool operator<(const CPayItem&);
   
   void  Clear();
   
   const double         GetAmount();
   const BhPayType      GetID();
   const BhPayTypeFlag  GetFlags();
   const char          *GetName();
   const BhDateTime&    GetTimestamp();
   void                 GetTimestamp(BhTimestamp *);
   const int            GetUser(){ return user; }
   const unsigned short GetSite(){ return site; }
   const unsigned short GetDept(){ return dept; }
   const unsigned short GetGroup(){ return group; }
   const unsigned short GetStation(){ return station; }
	const long GetAuth(){ return auth; }
   
   void  SetAmount(double);
   void  AddAmount(double);
   void  SetID(BhPayType);
   void  SetFlags(BhPayTypeFlag);
   bool  SetName(char *);
   void  SetTimestamp(BhDateTime&);
   void  SetUser(int i) { user=i; }
   void  SetWhere(char *);
   void  SetSite(unsigned short i) { site=i; }
   void  SetDept(unsigned short i) { dept=i; }
   void  SetGroup(unsigned short i) { group=i; }
   void  SetStation(unsigned short i) { station=i; }
	void  SetAuth(long i) { auth=i; }
};

/*! BhTransStatus
 \brief Status of the transaction
*/
enum BhTransStatus{
   BHTRANS_STATUS_UNKNOWN=0,
   BHTRANS_STATUS_IDLE,
   BHTRANS_STATUS_TRANSNUM,
   BHTRANS_STATUS_OPEN,
   BHTRANS_STATUS_TOTAL,
   BHTRANS_STATUS_TENDER,
   BHTRANS_STATUS_PAYAUTH,
   BHTRANS_STATUS_COMMIT,
   BHTRANS_STATUS_OPENFREE,
   BHTRANS_STATUS_OPENPICKEDUP,
   BHTRANS_STATUS_REOPENED,
   BHTRANS_STATUS_CLOSED
};

/*! BhTransNumber
 \brief Fetch a number
*/
enum BhTransNumber{
   BHNUM_SUBTOTAL,      /*! - subtotal*/
   BHNUM_TOTALITEMS,    /*! - total number of items*/
   BHNUM_TOTAL,         /*! - total*/
   BHNUM_DISCOUNT,      /*! - discount*/
   BHNUM_TAX,           /*! - tax*/
   BHNUM_TENDERED,      /*! - tendered*/
   BHNUM_CHANGE,        /*! - change*/
   BHNUM_TOTALTENDERED, /*! - total tendered*/
   BHNUM_CASHTENDERED,  /*! - cash tendered*/
   BHNUM_TOTALRETURN,   /*! - returns total*/
   BHNUM_LAYBYTOTAL,    /*! - laybys total*/
   BHNUM_LAYBYDEPOSIT,  /*! - layby deposits total*/
   BHNUM_RENTALTOTAL,   /*! - rentals total*/
   BHNUM_RENTALDEPOSIT, /*! - rentals deposit total*/
   BHNUM_ROUNDUNIT      /*! - currency rounding unit*/
};

enum BhTransEditType{
   BHTRANSEDITTYPE_QTY,
   BHTRANSEDITTYPE_PRICE
};

enum BhTransDiscountType{
   BHTRANS_DISCOUNT_NONE=0,
   BHTRANS_DISCOUNT_USER,
   BHTRANS_DISCOUNT_PROMO,
   BHTRANS_DISCOUNT_PRICEBREAK,
   BHTRANS_DISCOUNT_CUSTOMER
};
//! Transaction Round Type
enum BhTransRoundType{
   
   BHTRANS_ROUND_NONE=0, //! - no currency rounding 
   BHTRANS_ROUND_ROUND,  //! - round to nearest    
   BHTRANS_ROUND_UP,     //! - round up  
   BHTRANS_ROUND_DOWN    //! - round down  
};

/*! CReturnReason
 \brief A class to handle return reasons
*/
class CReturnReason 
{
private:
   int reason;
   int num;

public:
   CReturnReason();
   CReturnReason(int r, int n);
   ~CReturnReason();
   bool operator==(CReturnReason const &r);
   bool operator==(const int rid);
   bool operator!=(CReturnReason const &r);
   bool operator!=(const int tid);
   int operator++();
   int operator--();
   CReturnReason &operator=(CReturnReason const &r);
   int  GetReason() { return reason; }
   int  GetNum() { return num; }
   void SetReason(int i) { reason=i; }
   void SetNum(int i) { num=i; }
};

enum BhTransactionType{
   BHTRANSACTION_TYPE_NONE,
   BHTRANSACTION_TYPE_NEW,
   BHTRANSACTION_TYPE_CURRENT,
   BHTRANSACTION_TYPE_TRANSACTION,
   BHTRANSACTION_TYPE_DELIVERY,
   BHTRANSACTION_TYPE_EPHEMERAL,
   BHTRANSACTION_TYPE_RETURNS,
   BHTRANSACTION_TYPE_RETURNSPASSIVE,
   BHTRANSACTION_TYPE_RETRIEVE,
   BHTRANSACTION_TYPE_HISTORY
};

/*! CTransItems
 \brief A class to handle transactions
*/
class CTransItems 
{
protected:
   BhTransStatus status;
   BhDateTime    timestamp;
   BhTransDiscountType discount_type;
   BhTransactionType   type;
   int    ID;
   int    nameID;
   int    customerID;
   int    numunknown;
   int    numprice;
   int    numreturn;
   int    numlayby;
   int    numrental;
   int    numephemeral;
   int    numreturnreasons;
   int    numredemption;
   int    userid;
   char   userName[32];

   BhTransRoundType  currencyRound;
   double roundvalue;
   
   double total_tendered;
   double cash_tendered;
   double cash_change;

   double discount_margin;
   double total_discount;
   double total_tax;
   double total_items;
   double sub_total;
   double total;
   
   double total_return;
   double total_rental;
   double total_layby;
   double layby_deposit;
   double rental_deposit;
   
   std::list<CSaleItem>     items;
   std::list<CPayItem>      p_items;
   std::list<CReturnReason> rr_items;

public:
   CTransItems();
   CTransItems(const CTransItems& t);
   CTransItems(const int tid);
   virtual ~CTransItems();
   bool operator==(const CTransItems& t);
   bool operator==(const int tid);
   bool operator!=(const CTransItems& t);
   bool operator!=(const int tid);
   CTransItems &operator=(const CTransItems& t);
//   CTransItems *operator&();
   bool operator<(const CTransItems& t);
   int  operator+=(CTransItems& t);
   
   const BhTransStatus GetStatus(){ return status; }
   void  SetStatus(BhTransStatus s){ status=s; }
   
   const BhTransactionType GetType(){ return type; }
   void  SetType(BhTransactionType t){ type=t; }
   
   const BhDateTime& GetTimestamp();
   void              GetTimestamp(BhTimestamp *);
   void              SetTimestamp(BhDateTime&);
   
   int    Rationalize();
   int    UpdateAll();
   CTransItems &Updater(CSaleItem&);
   bool   Update(int n, CSaleItem& si);
   bool   Update(BhTransEditType t, int n, CSaleItem& si);
   bool   Update(CSaleItem&);
   bool   Update(CPayItem&);
   double Round(double amount);
	
	const int NumAllItemsQty();
	const int NumAllItems(){ return items.size(); }
   const int NumAllDispItems(){ return items.size() + numredemption; }
	const int NumItems() { return items.size() - (numunknown + numreturn + numlayby + numrental); }
	const int NumReturnItems() { return numreturn; }
   const int NumPriceItems() { return numprice; }
	const int NumUnknownItems() { return numunknown; }
	const int NumPayment() { return p_items.size(); }
	const int NumRentalItems() { return numrental; }
	const int NumLaybyItems() { return numlayby; }
	const int NumEphemeral() { return numephemeral; }
   const int NumReturnReasons() { return numreturnreasons; }
   const int NumPayItems() {return p_items.size(); }
   
   const CSaleItem& CurrItem(int seeker);
   
   /*! Finds an item by index.
		\param index the index in the list of the item to get
		\param item to be filled.
		\return 0 if item found.
	*/
   int GetItem(int index, CSaleItem& item);
   /*! Finds the first in item in a backward search.
		\param item the item to be filled, the barcode of the item should be set prior to calling this function
		\return 0 if item found.
	*/
   int GetItem(CSaleItem& item);
   int GetPayItem(int index, CPayItem& item);
   int GetPayItem(CPayItem& item);
   
   /*! Get a number from transaction.
		\param type The number to get
		\return value.
	*/
   const double GetNumber(BhTransNumber type);
   void         SetNumber(BhTransNumber type, double value);
   const int    GetID(){ return ID; }
   const int    GetCustomerID(){ return customerID; }
   const int    GetNameID(){ return nameID; }
   const int    GetReturnReason();
   
   BhTransDiscountType GetDiscountType() { return discount_type; }
   void SetDiscountType(BhTransDiscountType t) { discount_type=t; }
   
   void SetID(int i){ ID=i; }
   void SetNameID(int i){ nameID=i; }
   void SetCustomerID(int i){ customerID=i; }
   void SetUserID(int i){ userid=i; }
   void SetUserName(char *uname);
   int  GetUserID() { return userid; }
   char *GetUserName() { return userName; }
   int  Add(CSaleItem& item, bool bUpdate);
   int  Add(CSaleItem *item, bool bUpdate);
   int  Edit(CSaleItem& item);
   int  Edit(int, CSaleItem& item);
   int  Delete(CSaleItem& item);
   int  Delete(int index);
   int  Decrement(CSaleItem& item, int *);
   int  Exists(CSaleItem& item);
   int  Exists(int i, CSaleItem& si);
   int  Exists(CPayItem& item);
   int  Pop(CSaleItem& item);
   int  PopUnknown(CSaleItem& item);
   int  PopPrice(CSaleItem& item);
   int  PopRental(CSaleItem& item);
   int  PopLayby(CSaleItem& item);
   int  PopReturn(CSaleItem& item);
   int  PopEphemeral(CSaleItem& item);
   int  Push(CSaleItem& item);
   int  PaymentExists(CPayItem& item);
   int  AddPayment(CPayItem& item, bool bUpdate);
   int  AddReturnReason(int);
   int  DeletePayment(int index);
   int  DeletePayment(CPayItem& item);
   int  RemoveReturnReason(int);
   int  PopPayment(CPayItem& item);
   void Clear();
   void ClearPayItems();
   int  StoreTransaction();
   int  RetrieveTransaction();
   void SetRoundType(BhTransRoundType t);
};

enum BhItemType{
   BHITEM_TYPE_ALL,		// \enum retrieval descriptor - all
   BHITEM_TYPE_PLU,		// \enum retrieval descriptor - regular sale items
   BHITEM_TYPE_UPLU,		// \enum retrieval descriptor - unknown sale items
   BHITEM_TYPE_PRICE,	// \enum retrieval descriptor - cash sale items
   BHITEM_TYPE_RETURN,	// \enum retrieval descriptor - return sale items
   BHITEM_TYPE_LAYBY,	// \enum retrieval descriptor - layby sale items
   BHITEM_TYPE_RENTAL,  // \enum retrieval descriptor - rental sale items
   BHITEM_TYPE_QTY,   	// \enum retrieval descriptor - item quantity
   BHITEM_TYPE_DISP,    // \enum retrieval descriptor - item disp
   BHITEM_TYPE_PAYMENT, // \enum retrieval descriptor - item payment
   BHITEM_TYPE_EPHEM    // \enum retrieval descriptor - item ephemeral
};

/*! CTransStack
 \brief A class to handle a stack of transactions
*/
class CTransStack
{
private:
   int  current;
   int  old_current;
   bool bHistory;
   int  history_size;
   bool bBlankCycle;

protected:
   std::list<CTransItems> stack;
   std::list<CTransItems> old_stack;

public:
   CTransStack();
   CTransStack(CTransStack const &);
   ~CTransStack();
   CTransStack &operator=(CTransStack const &);
   bool operator!=(CTransStack const &);
   bool operator==(CTransStack const &);
   bool operator<(CTransStack const &);
   
   const int  GetIndex() { return current; }
   const int  GetTransID(int transid);
   const BhTransStatus GetStatus(int transid);
   const int  GetNameID(int transid);
   const int  NumTrans();
   const int  NumItems(int transid, BhItemType);
   const int  NumPayItems(int transid);
   
   const BhTransactionType GetType(int transid);
   void  SetType(int transid, BhTransactionType t);
   /*! \brief Set transaction stack to history mode.
      Sets the transaction stack to history mode, transactions fetched from
      history stack.
		\param b true to set history mode.
	*/
   void  SetHistory(bool b) { bHistory=b; }
   /*! \brief Set transaction history size.
      Sets the number of transactions to hold in the history stack.
		\param size stack size.
	*/
   void  SetHistorySize(int size) { history_size=size; }
   void  SetBlankCycle(bool b) { bBlankCycle=b; }
   bool  GetBlankCycle() { return bBlankCycle; }
   void  SetUserName(int transid, char *name);
   void  SetStatus(int transid, BhTransStatus);
   void  SetNameID(int transid, int nameid);
   bool  Exists(int transid);
   bool  Update(int transid, CTransItems&);
   bool  Update(int transid, CSaleItem&);
   bool  Update(int transid, int n, CSaleItem&);
   bool  Update(int transid, BhTransEditType t, int n, CSaleItem&);
   bool  Update(int transid, CPayItem&);
   bool  Get(int transid, CTransItems&);
   bool  GetHistory(int transid, CTransItems& item);
   /*! \brief Get next transaction in stack.
      Gets the next trancation in the stack, rotating to first after last.
      The passed CtransItems is filled from the current stack, or the historical
      stack if in historical mode.
		\param item to be filled.
		\return false if no item.
	*/
   int   GetNext(CTransItems& item);
   /*! \brief Add a blank transaction. */
   int   Add();
   /*! \brief Add a blank transaction with transaction id. */
   int   Add(int transid);
   /*! \brief Add a blank transaction with transaction id, and user name. */
   int   Add(int transid, int uid, char *uname);
   /*! \brief Add a CSaleItem to transaction.
   \param transid The id of the transaction.
   \param si CSaleItem.
   \param bUpdate Update the transaction values.
   \return 0 if no error.
   */
   int   AddItem(int transid, CSaleItem& si, bool bUpdate);
   /*! \brief Add a CSaleItem to transaction. */
   int   AddItem(int transid, CSaleItem * si, bool bUpdate);
   /*! \brief Add a CPayItem to transaction. */
   int   AddPayment(int transid, CPayItem&, bool bUpdate);
   /*! \brief Decrement quantity of a trans item.
   \param transid The id of the transaction.
   \param si CSaleItem.
   \param newquantity A passed pointer that will be filled with the new quantity. */
   int   Decrement(int transid, CSaleItem& si, int *newquantity);
   bool  Remove(int transid, CSaleItem&);
   bool  Remove(int transid, CPayItem&);
   const double GetNumber(int transid, BhTransNumber numbertype);
   void  SetNumber(int transid, BhTransNumber numbertype, double value);
   int   Pop(CTransItems& transitems);
   int   Push(CTransItems& transitems);
   void  Clear(int transid);
   void  ClearPayItems(int transid);
   void  ClearAll();
   int   Edit(CTransItems& old_Trans, CTransItems& new_trans);
   int   Delete(CTransItems& item);
};

enum BhSSTSNumber{
   BHSSTS_NUM_ALL,
   BHSSTS_NUM_OPEN,
   BHSSTS_NUM_LISTED,
   BHSSTS_NUM_PICKEDUP,
   BHSSTS_NUM_ASSIGNEDTO
};

/*! CServSideTrans
 \brief A server side transaction
*/
class CServSideTrans : public CTransItems
{
protected:
   Glib::ustring name;
   bool          bAll;
   int           creatorID;
   int           userID;
   int           stationID;
   int           targetID;
   Glib::ustring targetname;
   int           targetNum;
   int           languageID;
   int           interfaceID;

   BhSSTSNumber sstatus;
   
public:
   CServSideTrans();
   CServSideTrans(const int tn);
   CServSideTrans(const int tn,const int pid, const int uid);
   CServSideTrans(CServSideTrans const& t);
   CServSideTrans(char *, int);
   ~CServSideTrans();
   bool operator==(CServSideTrans const& t);
   bool operator!=(CServSideTrans const& t);
   CServSideTrans &operator=(CServSideTrans const& t);
   CServSideTrans *operator&();
   int operator-=(CServSideTrans const& t);
   int operator+=(CServSideTrans const& t);

   BhSSTSNumber GetSStatus(){ return sstatus; }
   const int    GetCreatorID() { return creatorID; }
   const int    GetUserID() { return userID; }
   const Glib::ustring GetName() { return name; }
   const Glib::ustring GetTargetName() { return targetname; }

   CTransItems::GetStatus;
   CTransItems::SetStatus;
   CTransItems::Rationalize;
   CTransItems::UpdateAll;
   CTransItems::Update;
	CTransItems::NumAllItemsQty;
	CTransItems::NumAllItems;
	CTransItems::NumItems;
	CTransItems::NumReturnItems;
   CTransItems::NumPriceItems;
	CTransItems::NumUnknownItems;
	CTransItems::NumPayment;
	CTransItems::NumRentalItems;
	CTransItems::NumLaybyItems;
	CTransItems::NumEphemeral;
   CTransItems::CurrItem;
   CTransItems::GetItem;
   CTransItems::GetPayItem;
   CTransItems::GetNumber;
   CTransItems::SetNumber;
   CTransItems::GetID;
   CTransItems::GetDiscountType;
   CTransItems::SetDiscountType;
   CTransItems::SetID;
   CTransItems::Add;
   CTransItems::Edit;
   CTransItems::Delete;
   CTransItems::Decrement;
   CTransItems::Exists;
   CTransItems::Pop;
   CTransItems::PopUnknown;
   CTransItems::PopPrice;
   CTransItems::PopRental;
   CTransItems::PopLayby;
   CTransItems::PopReturn;
   CTransItems::PopEphemeral;
   CTransItems::Push;
   CTransItems::PaymentExists;
   CTransItems::AddPayment;
   CTransItems::DeletePayment;
   CTransItems::PopPayment;
   CTransItems::Clear;
   CTransItems::ClearPayItems;
   CTransItems::StoreTransaction;
   CTransItems::RetrieveTransaction;
   /*
   const int NumItems() { return items.size(); }
   const int NumUnknownItems() { return numunknown; }
   const int NumPayment() { return p_items.size(); }
   const CSaleItem& CurrItem(int seeker);
   */
};

enum BhTargetStatus{
   BHTARGET_STATUS_UNKNOWN=0,
   BHTARGET_STATUS_OPEN,
   BHTARGET_STATUS_RESERVED,
   BHTARGET_STATUS_INUSE,
   BHTARGET_STATUS_NEAREND
};

/*! CServSideTarget
 \brief A server side transaction target
*/
class CServSideTarget
{
protected:
   int           ID;
   Glib::ustring name;
   int           capacity;
   int           current;
   int           userID;
   //time_t   start_time;
   BhTargetStatus status;
   
public:
   CServSideTarget();
   CServSideTarget(const int tn);
   CServSideTarget(const int tn, const char *n, const int c);
   CServSideTarget(CServSideTarget const& t);
   ~CServSideTarget();
   bool operator==(CServSideTarget const& t);
   bool operator!=(CServSideTarget const& t);
   CServSideTarget &operator=(CServSideTarget const& t);
   int operator-=(CServSideTarget const& t);
   int operator+=(CServSideTarget const& t);
   void SetStatus(BhTargetStatus s);
   BhTargetStatus GetStatus();
};

/*! CServSideProductOption
 \brief A server side product option
*/
class CServSideProductOption
{
protected:
   int   ID;
   char  description[64];
   char  ReceiptText[32];
   
public:
   CServSideProductOption();
   CServSideProductOption(const int tn);
   CServSideProductOption(const int tn, const char *d, const char *r);
   CServSideProductOption(CServSideProductOption const& t);
   ~CServSideProductOption();
   bool operator==(CServSideProductOption const& t);
   bool operator!=(CServSideProductOption const& t);
   CServSideProductOption &operator=(CServSideProductOption const& t);
};

/*! CServSideProduct
 \brief A server side product
*/
class CServSideProduct
{
protected:
   int           ID;
   bool          bAvailable;
   Glib::ustring name;
   Glib::ustring description;
   float         price;
   int           printerID;
   Glib::ustring ReceiptText;
   
   std::list<CServSideProductOption> options;

public:
   CServSideProduct();
   CServSideProduct(const int tn);
   CServSideProduct(const int t, const char *n, const char *d, const char *r,int pid, double p);
   CServSideProduct(CServSideProduct const& t);
   ~CServSideProduct();
   bool operator==(CServSideProduct const& t);
   bool operator!=(CServSideProduct const& t);
   CServSideProduct &operator=(CServSideProduct const& t);

   int AddOption(CServSideProductOption&);
};

/*! CServSideMenu
 \brief A server side menu
*/
class CServSideMenu
{
protected:
   int           ID;
   Glib::ustring name;
   Glib::ustring description;
   
   std::list<CServSideProduct> menu;

public:
   CServSideMenu();
   CServSideMenu(const int tn);
   //CServSideMenu(const int, const char *, const char *);
   CServSideMenu(CServSideMenu const& t);
   ~CServSideMenu();
   bool operator==(CServSideMenu const& t);
   bool operator!=(CServSideMenu const& t);
   CServSideMenu &operator=(CServSideMenu const& t);

   int Add(CServSideProduct&);
};

/*! CServSideTransStack
 \brief A server side transaction stack
*/
class CServSideTransStack
{
protected:
   std::list<CServSideTrans>  stack;
   std::list<CServSideTarget> targets;

   CServSideMenu menu;

public:
   CServSideTransStack();
   CServSideTransStack(CServSideTransStack const&);
   ~CServSideTransStack();
   /*
   bool operator==(CServSideTransStack const&);
   void operator=(CServSideTransStack const&);
   int operator-=(CServSideTransStack const&);
   int operator+=(CServSideTransStack const&);
   */
   int NumTransactions(BhSSTSNumber, int);
   //int List(BhSSTSNumber, int, char *, char *, int *);
   int TargetList(BhSSTSNumber, int, BhTransRoverList *);
   int List(BhSSTSNumber, int, BhTransRoverList *);
   
   const int  NumTrans(){ return stack.size(); }
   const int  NumItems(int, BhItemType);
   
   bool Exists(int tid);
   bool Update(int tid, CServSideTrans&);
   bool Get(int tid, CServSideTrans&);
   int  Add(int tid, int pid, int uid);
   int  AddItem(int tid, CSaleItem&, bool);
   int  AddPayment(int tid, CPayItem&, bool);
   void SetNumber(int tid, BhTransNumber, double);
   int  Pop(CServSideTrans&);
   int  Push(CServSideTrans&);
   void Clear(int tid);
   void ClearAll();
   int  Edit(CServSideTrans&, CServSideTrans&);
   int  Delete(CServSideTrans&);
   int  Head(CServSideTrans&);
   int  Tail(CServSideTrans&);
   int  Next(CServSideTrans&);
   int  Prev(CServSideTrans&);
   
   int  AddTarget(CServSideTarget&);
   int  AddMenu(CServSideMenu&);
   int  NumOpenTargets();
};

/* @} 
	@} */

}

#endif // _CTRANSITEM_
