/*
ctransstack.cpp - a transaction stack.

Copyright (C) 2002 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2004 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 
*/
#include <pthread.h>
#include <cstdio>
#include <cstdlib>
#include <fcntl.h>
#include <unistd.h>
#include "ctransitems.h"

namespace bhpos{

CTransStack::CTransStack(CTransStack const& ts)
{
	stack=ts.stack;
   old_stack=ts.old_stack;
   current=ts.current;
   old_current=ts.old_current;
   bHistory=ts.bHistory;
   history_size=ts.history_size;
   bBlankCycle=ts.bBlankCycle;
}
CTransStack::CTransStack()
{
   current=0;
   old_current=0;
   bHistory=false;
   history_size=0;
   bBlankCycle=true;
}

CTransStack::~CTransStack()
{
	stack.clear();
   old_stack.clear();
}

CTransStack &CTransStack::operator=(CTransStack const& ts)
{
	this->stack=ts.stack;
   this->old_stack=ts.old_stack;
	this->current=ts.current;
   this->old_current=ts.old_current;
   this->bHistory=ts.bHistory;
   this->history_size=ts.history_size;
   this->bBlankCycle=ts.bBlankCycle;
   
	return *this;
	/*
	std::list<CTransItems>::iterator i;
	
	if(!ts.stack.empty()){
		for(i=ts.stack.begin(); i!=ts.stack.end(); i++){
			stack.push_back(*i);
		}
	}*/
}

bool CTransStack::operator==(CTransStack const& ts)
{
	if(stack.size()==ts.stack.size()) return true;
	return false;
}

bool CTransStack::operator!=(CTransStack const& ts)
{
	if(stack.size()!=ts.stack.size()) return true;
	return false;
}

bool CTransStack::operator<(CTransStack const& ts)
{
	if(stack.size() < ts.stack.size()) return true;
	return false;
}

const int CTransStack::GetTransID(int index)
{
   std::list<CTransItems>::iterator i;
   
   if(bHistory) i=old_stack.begin();
   else i=stack.begin();
	int x=index;
   
   if(bHistory){
      if(old_stack.size()==0) return -1;
   }
   else{
      if(stack.size()==0) return -1;
   }
   
   while(x){
      i++;
      x--;
   }
   if(bHistory){
      if(i==old_stack.end()) return -1;
   }
   else{
      if(i==stack.end()) return -1;
   }      
      
	return i->GetID();
}

const int CTransStack::NumTrans()
{
   if(bHistory){
      return old_stack.size(); 
   }
   else{
      return stack.size(); 
   }
}

const int CTransStack::NumItems(int tid, BhItemType type)
{
	CTransItems t;
	if(Get(tid, t)){
	   switch(type){
	   case BHITEM_TYPE_ALL: return t.NumAllItems();
	   case BHITEM_TYPE_PLU: return t.NumItems();
      case BHITEM_TYPE_UPLU: return t.NumUnknownItems();
      case BHITEM_TYPE_RETURN: return t.NumReturnItems();
      case BHITEM_TYPE_LAYBY: return t.NumLaybyItems();
      case BHITEM_TYPE_RENTAL:  return t.NumRentalItems();
      case BHITEM_TYPE_QTY:  return t.NumAllItemsQty();
      case BHITEM_TYPE_DISP: return t.NumAllDispItems();
      case BHITEM_TYPE_PRICE: return t.NumPriceItems();
      case BHITEM_TYPE_PAYMENT: return t.NumPayment();
      case BHITEM_TYPE_EPHEM: return t.NumEphemeral();
      }
	}
	
	return -1;
}

const int CTransStack::NumPayItems(int tid)
{
	std::list<CTransItems>::iterator i=find(stack.begin(), stack.end(), tid);
	
   if(i!=stack.end()){
      return i->NumPayItems();
   }
	
	return 0;
}

const BhTransStatus CTransStack::GetStatus(int tid)
{
   std::list<CTransItems>::iterator i;
   
   if(bHistory){
      i=find(old_stack.begin(), old_stack.end(), tid);
      if(i!=old_stack.end()){
         return i->GetStatus();
      }
   }
   else{
      i=find(stack.begin(), stack.end(), tid);
      if(i!=stack.end()){
         return i->GetStatus();
      }
   }
	
   return BHTRANS_STATUS_UNKNOWN;
}

void CTransStack::SetStatus(int tid, BhTransStatus s)
{
   std::list<CTransItems>::iterator i;
   
   if(bHistory){
      i=find(old_stack.begin(), old_stack.end(), tid);
	   if(i!=old_stack.end()){
         return i->SetStatus(s);
      }
   }
   else{
      i=find(stack.begin(), stack.end(), tid);
	   if(i!=stack.end()){
         return i->SetStatus(s);
      }
   }
}

const BhTransactionType CTransStack::GetType(int tid)
{ 
   std::list<CTransItems>::iterator i;
   
   if(bHistory){
      i=find(old_stack.begin(), old_stack.end(), tid);
	   if(i!=old_stack.end()){
         return i->GetType();
      }
   }
   else{
      i=find(stack.begin(), stack.end(), tid);
	   if(i!=stack.end()){
         return i->GetType();
      }
   }
   return BHTRANSACTION_TYPE_NONE;
}

void  CTransStack::SetType(int tid, BhTransactionType t)
{
   std::list<CTransItems>::iterator i;
   
   if(bHistory){
      i=find(old_stack.begin(), old_stack.end(), tid);
	   if(i!=old_stack.end()){
         return i->SetType(t);
      }
   }
   else{
      i=find(stack.begin(), stack.end(), tid);
	   if(i!=stack.end()){
         return i->SetType(t);
      }
   }
}

const int CTransStack::GetNameID(int tid)
{
   std::list<CTransItems>::iterator i;
   
   if(bHistory){
      i=find(old_stack.begin(), old_stack.end(), tid);
	   if(i!=old_stack.end()){
         return i->GetNameID();
      }
   }
   else{
      i=find(stack.begin(), stack.end(), tid);
	   if(i!=stack.end()){
         return i->GetNameID();
      }
   }
   return 1;
}

void CTransStack::SetNameID(int tid, int nid)
{
   std::list<CTransItems>::iterator i;
   if(bHistory){
      i=find(old_stack.begin(), old_stack.end(), tid);
	   if(i!=old_stack.end()){
         return i->SetNameID(nid);
      }
   }
   else{
      i=find(stack.begin(), stack.end(), tid);
	   if(i!=stack.end()){
         return i->SetNameID(nid);
      }
   }
}

void CTransStack::SetUserName(int tid, char *name)
{
   std::list<CTransItems>::iterator i;
   if(bHistory){
   }
   else{
      i=find(stack.begin(), stack.end(), tid);
	   if(i!=stack.end()){
         i->SetUserName(name);
      }
   }
}

void CTransStack::Clear(int tid)
{
	std::list<CTransItems>::iterator i;
   
   if(bHistory){
      i=find(old_stack.begin(), old_stack.end(), tid);
	   if(i!=old_stack.end()) old_stack.erase(i);
   }
   else{
      i=find(stack.begin(), stack.end(), tid);
	   if(i!=stack.end()) stack.erase(i);
   }
}

void CTransStack::ClearPayItems(int tid)
{
   std::list<CTransItems>::iterator i=find(stack.begin(), stack.end(), tid);
	
   if(i!=stack.end()){
      i->ClearPayItems();
   }
}

void CTransStack::ClearAll()
{
   stack.clear();
   old_stack.clear();
}

const double CTransStack::GetNumber(int tid, BhTransNumber n)
{
   CTransItems t;
   
	if(Get(tid, t)){
      return t.GetNumber(n);
   }
   return 0.0;
}

void CTransStack::SetNumber(int tid, BhTransNumber n, double v)
{
   CTransItems t;
   
	if(Get(tid, t)){
      t.SetNumber(n, v);
      Update(tid, t);
   }
}

bool CTransStack::Update(int tid, CTransItems& ti)
{
	std::list<CTransItems>::iterator i;
   
   if(bHistory){
      i=find(old_stack.begin(), old_stack.end(), tid);
	   if(i!=old_stack.end()){
		   *i=ti;
		   return true;
      }
	}
   else{
      i=find(stack.begin(), stack.end(), tid);
	   if(i!=stack.end()){
		   *i=ti;
		   return true;
      }
   }
	return false;
}

bool CTransStack::Update(int tid, CSaleItem& si)
{
	std::list<CTransItems>::iterator i;
   
   if(bHistory){
      i=find(old_stack.begin(), old_stack.end(), tid);
	   if(i!=old_stack.end()){
		   i->Update(si);
		   return true;
      }
	}
   else{
      i=find(stack.begin(), stack.end(), tid);
	   if(i!=stack.end()){
		   i->Update(si);
		   return true;
      }
   }
   
	return false;
}

bool CTransStack::Update(int transid, int n, CSaleItem& si)
{
   std::list<CTransItems>::iterator i;
   
   if(bHistory){
      i=find(old_stack.begin(), old_stack.end(), transid);
	   if(i!=old_stack.end()){
		   i->Update(n, si);
		   return true;
      }
	}
   else{
      i=find(stack.begin(), stack.end(), transid);
	   if(i!=stack.end()){
		   i->Update(n, si);
		   return true;
      }
   }
   
	return false;
}

bool CTransStack::Update(int transid, BhTransEditType t, int n, CSaleItem& si)
{
   std::list<CTransItems>::iterator i;
   
   if(bHistory){
      i=find(old_stack.begin(), old_stack.end(), transid);
	   if(i!=old_stack.end()){
		   i->Update(t, n, si);
		   return true;
      }
	}
   else{
      i=find(stack.begin(), stack.end(), transid);
	   if(i!=stack.end()){
		   i->Update(t, n, si);
		   return true;
      }
   }
   
	return false;
}

bool CTransStack::Update(int tid, CPayItem& pi)
{
	std::list<CTransItems>::iterator i;
   
   if(bHistory){
      i=find(old_stack.begin(), old_stack.end(), tid);
	   if(i!=old_stack.end()){
		   i->Update(pi);
		   return true;
	   }
   }
   else{
      i=find(stack.begin(), stack.end(), tid);
	   if(i!=stack.end()){
		   i->Update(pi);
		   return true;
	   }
   }
	return false;
}

bool CTransStack::Exists(int tid)
{
	std::list<CTransItems>::iterator i;
	
   if(bHistory){
      if(!old_stack.empty()){
         for(i=old_stack.begin(); i!=old_stack.end(); i++){
            if(i->GetID()==tid){
               return true;
            }
         }
      }
   }
   else{
      if(!stack.empty()){
         for(i=stack.begin(); i!=stack.end(); i++){
            if(i->GetID()==tid){
               return true;
            }
         }
      }
   }
   return false;
}

bool CTransStack::Get(int tid, CTransItems& ti)
{
   std::list<CTransItems>::iterator i;
	
   if(bHistory){
      if(!old_stack.empty()){
         for(i=old_stack.begin(); i!=old_stack.end(); i++){
            if(i->GetID()==tid){
               ti=*i;
               return true;
            }
         }
      }
   }
   else{
      if(!stack.empty()){
         for(i=stack.begin(); i!=stack.end(); i++){
            if(i->GetID()==tid){
               ti=*i;
               return true;
            }
         }
      }
   }
   return false;
}

bool CTransStack::GetHistory(int tid, CTransItems& ti)
{
   std::list<CTransItems>::iterator i;
	
   if(!old_stack.empty()){
      for(i=old_stack.begin(); i!=old_stack.end(); i++){
         if(i->GetID()==tid){
            ti=*i;
            return true;
         }
      }
   }
   return false;
}

int CTransStack::GetNext(CTransItems& ti)
{
   std::list<CTransItems>::iterator i;
   int n;
   
   if(bHistory){	
      if(old_stack.empty()){
         return -2;
      }
      old_current++;
      if(old_current >= old_stack.size()){
         old_current=0;
      }
      n=old_current;
      i=old_stack.begin();
      while(n){
         i++;
         n--;
      }
      ti=*i;
      return old_current;
   }
   else{
      if(stack.empty()){
         return -2;
      }
      current++;
      
      if(current >= stack.size()){
         if(bBlankCycle){
            current=-1;
            return -1;
         }
         current=0;
      }
      n=current;
      
      i=stack.begin();
      while(n){
         i++;
         n--;
      }
      ti=*i;
      return current;
   }
   return -2;
}

int CTransStack::Add()
{
	CTransItems t;
	
	stack.push_back(t);
	
	return stack.size();
}

int CTransStack::Add(int tid)
{
	CTransItems t(tid);
	
	stack.push_back(t);
	
	return stack.size();
}

int CTransStack::Add(int tid, int uid, char *uname)
{
	CTransItems t(tid);
	t.SetUserName(uname);
   t.SetUserID(uid);
	stack.push_back(t);
	
	return stack.size();
}

int CTransStack::AddItem(int tid, CSaleItem& si, bool bUpdate)
{
	int			ret=0;
	CTransItems t;
	
	Get(tid, t);
	ret=t.Add(si, bUpdate);
	if(bUpdate) Update(tid, t);	
	return ret;
}

int CTransStack::AddItem(int tid, CSaleItem *si, bool bUpdate)
{
	int			ret=0;
	CTransItems t;
   CSaleItem   s=*si;
	
	Get(tid, t);
	ret=t.Add(s, bUpdate);
	if(bUpdate) Update(tid, t);	
	return ret;
}

int CTransStack::AddPayment(int tid, CPayItem& pi, bool bUpdate)
{
   int			ret=0;
   CTransItems t;
	
   Get(tid, t);
   ret=t.AddPayment(pi, bUpdate);
   Update(tid, t);
   return ret;
}
	
int CTransStack::Pop(CTransItems& ti)
{
   ti=stack.back();
   if(history_size!=0){
      ti.SetType(BHTRANSACTION_TYPE_HISTORY);
      old_stack.push_front(ti);
      if(old_stack.size() >= history_size){
         old_stack.pop_back();
      }
   }
   stack.pop_back();
   
   return 0;
}

int CTransStack::Push(CTransItems& ti)
{
   stack.push_back(ti);
   return 0;
}

int CTransStack::Edit(CTransItems& tio, CTransItems& tin)
{
	std::list<CTransItems>::iterator i;
	
	if(!stack.empty()){
		for(i=stack.begin(); i!=stack.end(); i++){
			if(*i==tio){
				*i=tin;
				return 0;
			}
		}
	}
	
	return 1;
}

bool CTransStack::Remove(int tid, CSaleItem& si)
{
	std::list<CTransItems>::iterator i=find(stack.begin(), stack.end(), tid);
	if(i!=stack.end()){
		i->Delete(si);
		return true;
	}
	return false;
}

bool CTransStack::Remove(int tid, CPayItem& pi)
{
	std::list<CTransItems>::iterator i=find(stack.begin(), stack.end(), tid);
	if(i!=stack.end()){
		i->DeletePayment(pi);
		return true;
	}
	return false;
}

int CTransStack::Decrement(int tid, CSaleItem& item, int *nq)
{
   std::list<CTransItems>::iterator i=find(stack.begin(), stack.end(), tid);
	if(i!=stack.end()){
		i->Decrement(item, nq);
		return true;
	}
	return false;
}

int CTransStack::Delete(CTransItems& ti)
{
	std::list<CTransItems>::iterator i=find(stack.begin(), stack.end(), ti);
	if(i!=stack.end()) stack.erase(i);
	return 0;
}

}
